from aenum import Enum


class JobRunLogStatuses(Enum):
    """This class is an enum of all the JobRunLogStatuses supported by the Pantomath Python SDK."""

    STARTED = "STARTED"
    IN_PROGRESS = "IN_PROGRESS"
    SUCCEEDED = "SUCCEEDED"
    FAILED = "FAILED"
    QUEUED = "QUEUED"
    WARNING = "WARNING"

    @staticmethod
    def get_job_run_log_statuses():
        """Get a list of all the enum values
        ...
        :return: list of all the enum values
        :rtype: list of strings
        """
        return dir(JobRunLogStatuses)[: len(JobRunLogStatuses)]

    @staticmethod
    def is_job_run_log_status(input):
        """Validates if the unput is part of the enum
        :param input: string to check if part of enum
        :type input: str
        ...
        :return: True if in enum else False
        :rtype: Boolean
        """
        return input in JobRunLogStatuses.get_job_run_log_statuses()



from pantomath_sdk import (
    TableauExtractRefreshTask,
    PlatformTypes,
    AssetPathTypes,
    Asset,
)


def test_tableau_extract_refresh_task_good():
    node = TableauExtractRefreshTask(
        name="TableauExtractRefreshTask Unit Test",
        site_id="site_id",
        refresh_id="refresh_id",
        host="host",
    )
    assert node.get_name() == "TableauExtractRefreshTask Unit Test"
    assert node.get_type() == "TABLEAU_EXTRACT_REFRESH_TASK"
    assert node.get_fully_qualified_object_name() == "host/site/site_id/task/refresh_id"
    assert node.get_platform_type() == PlatformTypes.TABLEAU.value
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "TABLEAU", "type": "PLATFORM"}, {"depth": 1, "name": "Tableau Connection", "type": "CONNECTION"}]'
    )


def test_custom_tableau_extract_refresh_task_asset_path():
    node = TableauExtractRefreshTask(
        name="TableauExtractRefreshTask Unit Test",
        site_id="site_id",
        refresh_id="refresh_id",
        host="host",
        assets=[Asset(name="Workbook", type=AssetPathTypes.WORKFLOW.value)],
    )
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "TABLEAU", "type": "PLATFORM"}, {"depth": 1, "name": "Tableau Connection", "type": "CONNECTION"}, {"depth": 2, "name": "Workbook", "type": "WORKFLOW"}]'
    )



from pantomath_sdk import (
    FivetranConnectorConstructor,
    PlatformTypes,
    Asset,
    AssetPathTypes,
)


def test_fivetran_connector_constructor_good():
    node = FivetranConnectorConstructor(
        name="FivetranConnectorConstructor Unit Test",
    )
    assert node.get_name() == "FivetranConnectorConstructor Unit Test"
    assert node.get_type() == "FIVETRAN_CONNECTOR"
    assert (
        node.get_fully_qualified_object_name()
        == "FivetranConnectorConstructor Unit Test"
    )
    assert node.get_platform_type() == PlatformTypes.FIVETRAN.value
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "FIVETRAN", "type": "PLATFORM"}, {"depth": 1, "name": "Fivetran Connection", "type": "CONNECTION"}]'
    )


def test_custom_fivetran_connector_asset_path_good():
    node = FivetranConnectorConstructor(
        name="FivetranConnectorConstructor Unit Test",
        assets=[
            Asset("Factory", AssetPathTypes.FACTORY.value),
        ],
    )
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "FIVETRAN", "type": "PLATFORM"}, {"depth": 1, "name": "Fivetran Connection", "type": "CONNECTION"}, {"depth": 2, "name": "Factory", "type": "FACTORY"}]'
    )



from pantomath_sdk import AWSLambda, PlatformTypes, Asset, AssetPathTypes


def test_aws_lambda_good():
    node = AWSLambda(
        name="AWSLambda Unit Test",
    )
    assert node.get_name() == "AWSLambda Unit Test"
    assert node.get_type() == "AWS_LAMBDA"
    assert node.get_fully_qualified_object_name() == "AWSLambda Unit Test"
    assert node.get_platform_type() == PlatformTypes.AWS.value

    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "AWS", "type": "PLATFORM"}, {"depth": 1, "name": "AWS Connection", "type": "CONNECTION"}]'
    )


def test_custom_aws_lambda_asset_path():
    node = AWSLambda(
        name="AWSLambda Unit Test",
        assets=[
            Asset(name="Organization A", type=AssetPathTypes.ORGANIZATION.value),
            Asset(name="Container A", type=AssetPathTypes.CONTAINER.value),
        ],
    )

    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "AWS", "type": "PLATFORM"}, {"depth": 1, "name": "AWS Connection", "type": "CONNECTION"}, {"depth": 2, "name": "Organization A", "type": "ORGANIZATION"}, {"depth": 3, "name": "Container A", "type": "CONTAINER"}]'
    )



import pantomath_sdk as util


def test_normalizeHost():
    assert util.normalize_host("test/") == "test"


def test_sanitize_string():
    assert util.sanitize_string(" ") == "%20"
    assert util.sanitize_string(" \t") == "%20%09"



from pantomath_sdk.model.dataset.dataset import DataSet
from pantomath_sdk.enums.dataset_type import DataSetTypes
from pantomath_sdk.enums.platform_types import PlatformTypes


class SqlView(DataSet):
    """SqlView's DataSet Class used for getting the required infomation for Pantomath
        :param host: The SQL Host Url
        :type host: str
        :param port: Port number
        :type port: str
        :param database: Name of the database
        :type database: str
        :param schema: Name of the view's schema
        :type schema: str
        :param name: Name of the View
        :type name: str
    ...
    """

    def __init__(self, host, port, database, schema, name):
        """Constructor method"""
        self._host = host
        self._port = port
        self._database = database
        self._schema = schema
        self._name = name

    @staticmethod
    def create(host, port, database, schema, name):
        """Static method for obtaining SqlView's DataSet Class
        used for getting the required infomation for Pantomath
        :param host: The SQL Host Url
        :type host: str
        :param port: Port number
        :type port: str
        :param database: Name of the database
        :type database: str
        :param schema: Name of the view's schema
        :type schema: str
        :param name: Name of the View
        :type name: str
        ...
        :return: SqlView class object
        :rtype: SqlView
        """
        return SqlView(host, port, database, schema, name)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return DataSetTypes.SQL_VIEW.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return (
            self._host
            + ":"
            + str(self._port)
            + "."
            + self._database
            + "."
            + self._schema
            + "."
            + self._name
        )

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.SQL_SERVER.value



from aenum import Enum


class JobTriggerTypes(Enum):
    """This class is an enum of all the JobTriggerTypes supported by the Pantomath Python SDK."""

    SCHEDULE = "SCHEDULE"
    MANUAL = "MANUAL"
    UNKNOWN = "UNKNOWN"

    @staticmethod
    def get_job_trigger_types():
        """Get a list of all the enum values
        ...
        :return: list of all the enum values
        :rtype: list of strings
        """
        return dir(JobTriggerTypes)[: len(JobTriggerTypes)]

    @staticmethod
    def is_job_trigger_type(input):
        """Validates if the unput is part of the enum
        :param input: string to check if part of enum
        :type input: str
        ...
        :return: True if in enum else False
        :rtype: Boolean
        """
        return input in JobTriggerTypes.get_job_trigger_types()



from aenum import Enum


class JobObjectTypes(Enum):
    """This class is an enum of all the Jobs supported by the Pantomath Python SDK."""

    SQL_FUNCTION = "SQL_FUNCTION"
    SQL_PROCEDURE = "SQL_PROCEDURE"
    TABLEAU_EXTRACT_REFRESH_TASK = "TABLEAU_EXTRACT_REFRESH_TASK"
    FIVETRAN_CONNECTOR = "FIVETRAN_CONNECTOR"
    DBT_MODEL = "DBT_MODEL"
    POWER_BI_REFRESH = "POWER_BI_REFRESH"
    POWER_BI_ACTIVITY = "POWER_BI_ACTIVITY"
    POWER_BI_DATAFLOW = "POWER_BI_DATAFLOW"
    ADF_ACTIVITY = "ADF_ACTIVITY"
    ADF_PIPELINE = "ADF_PIPELINE"
    JOB = "JOB"
    ACTIVITY = "ACTIVITY"
    DATAFLOW = "DATAFLOW"
    PIPELINE = "PIPELINE"
    PACKAGE = "PACKAGE"
    COMPONENT = "COMPONENT"
    REFRESH = "REFRESH"
    TASK = "TASK"
    SNOWFLAKE_PIPE = "SNOWFLAKE_PIPE"

    @staticmethod
    def get_job_object_types():
        """Get a list of all the enum values
        ...
        :return: list of all the enum values
        :rtype: list of strings
        """
        return dir(JobObjectTypes)[: len(JobObjectTypes)]

    @staticmethod
    def is_job_object_type(input):
        """Validates if the unput is part of the enum
        :param input: string to check if part of enum
        :type input: str
        ...
        :return: True if in enum else False
        :rtype: Boolean
        """
        return input in JobObjectTypes.get_job_object_types()



from pantomath_sdk import PowerBiDataFlow, PlatformTypes
from pantomath_sdk.enums.asset_path import AssetPathTypes
from pantomath_sdk.model.asset_path.asset_path import Asset


def test_power_bi_data_flow_good():
    node = PowerBiDataFlow(
        name="PowerBiDataFlow Unit Test",
        group_id="245632457634",
        object_id="5785678976809",
    )
    assert node.get_name() == "PowerBiDataFlow Unit Test"
    assert node.get_type() == "POWER_BI_DATAFLOW"
    assert (
        node.get_fully_qualified_object_name()
        == "app.powerbi.com/groups/245632457634/dataflows/5785678976809"
    )
    assert node.get_platform_type() == PlatformTypes.POWERBI.value
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "POWERBI", "type": "PLATFORM"}, {"depth": 1, "name": "PowerBI Connection", "type": "CONNECTION"}]'
    )


def test_custom_power_bi_dataflow_asset_path_good():
    node = PowerBiDataFlow(
        name="PowerBiDataFlow Unit Test",
        group_id="245632457634",
        object_id="5785678976809",
        assets=[
            Asset(name="Factory", type=AssetPathTypes.FACTORY.value),
        ],
    )

    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "POWERBI", "type": "PLATFORM"}, {"depth": 1, "name": "PowerBI Connection", "type": "CONNECTION"}, {"depth": 2, "name": "Factory", "type": "FACTORY"}]'
    )



from pantomath_sdk import TableauDatasource, PlatformTypes


def test_tableau_datasource_good():
    test_node = TableauDatasource(
        host="host",
        uri="uri",
        name="TableauDatasource Unit Test",
    )
    assert test_node.get_name() == "TableauDatasource Unit Test"
    assert test_node.get_type() == "TABLEAU_DATASOURCE"
    assert test_node.get_fully_qualified_object_name() == "host/uri"
    assert test_node.get_platform_type() == PlatformTypes.TABLEAU.value



from aenum import Enum


class AssetPathTypes(Enum):
    """This class is an enum of all the AssetPathTypes supported by the Pantomath Python SDK."""

    ACCOUNT = "ACCOUNT"
    AGENT = "AGENT"
    APP = "APP"
    APP_HOST = "APP_HOST"
    BUCKET = "BUCKET"
    CATALOG = "CATALOG"
    CHANNEL = "CHANNEL"
    CONNECTION = "CONNECTION"
    CONNECTOR = "CONNECTOR"
    CONTAINER = "CONTAINER"
    COUNTER = "COUNTER"
    CUBE = "CUBE"
    DAG = "DAG"
    DASHBOARD = "DASHBOARD"
    DATABASE = "DATABASE"
    DATAFLOW = "DATAFLOW"
    DATASET = "DATASET"
    DATASOURCE = "DATASOURCE"
    DEPLOYMENT = "DEPLOYMENT"
    DIMENSION = "DIMENSION"
    ENVIRONMENT = "ENVIRONMENT"
    FACTORY = "FACTORY"
    FLOW = "FLOW"
    FOLDER = "FOLDER"
    FUNCTION = "FUNCTION"
    GROUP = "GROUP"
    HOST = "HOST"
    HUB = "HUB"
    JOB = "JOB"
    MEASURE = "MEASURE"
    MEASURE_GROUP = "MEASURE_GROUP"
    MODEL = "MODEL"
    OBJECT = "OBJECT"
    ORGANIZATION = "ORGANIZATION"
    PACKAGE = "PACKAGE"
    PARTITION = "PARTITION"
    PERSPECTIVE = "PERSPECTIVE"
    PIPELINE = "PIPELINE"
    PLAN = "PLAN"
    PLATFORM = "PLATFORM"
    PROJECT = "PROJECT"
    RECIPE = "RECIPE"
    REFRESH = "REFRESH"
    REGION = "REGION"
    REPORT = "REPORT"
    RESOURCE_GROUP = "RESOURCE_GROUP"
    ROUTINE = "ROUTINE"
    SCENARIO = "SCENARIO"
    SCHEDULE = "SCHEDULE"
    SCHEDULER = "SCHEDULER"
    SCHEMA = "SCHEMA"
    SERVER = "SERVER"
    SITE = "SITE"
    STAGE = "STAGE"
    STORAGE_ACCOUNT = "STORAGE_ACCOUNT"
    SUBSCRIPTION = "SUBSCRIPTION"
    TABLE = "TABLE"
    TASK = "TASK"
    TENANT = "TENANT"
    URI = "URI"
    WORKFLOW = "WORKFLOW"
    WORKSPACE = "WORKSPACE"
    ZONE = "ZONE"

    @staticmethod
    def get_asset_path_types():
        """Get a list of all the enum values
        ...
        :return: list of all the enum values
        :rtype: list of strings
        """
        return dir(AssetPathTypes)[: len(AssetPathTypes)]

    @staticmethod
    def is_object_type(input):
        """Validates if the unput is part of the enum
        :param input: string to check if part of enum
        :type input: str
        ...
        :return: True if in enum else False
        :rtype: Boolean
        """
        return input in AssetPathTypes.get_asset_path_types()



import uuid
from pantomath_sdk.model.job.job import Job
import datetime


class JobRunLog(Job):
    """JobRunLog's Job Class used for getting the required infomation for Pantomath
    :param job_run: Job Run to be logged
    :type job_run: JobRun
    :param status: Status of the Job Run
    :type status: JobRunLogStatuses
    :param message: Message for the log, defaults to empty string
    :type message: str
    :param records_effected: Number of records effected, defaults to None
    :type records_effected: number
    :param timestamp: Timestamp of the time the job ran, defaults to current UTC time
    :type timestamp: datetime
    ...
    """

    def __init__(
        self,
        job_run,
        status,
        message="",
        records_effected=None,
        timestamp=None,
    ):
        """Constructor method"""
        self._unique_id = uuid.uuid4()
        self._job_run = job_run
        self._status = status
        self._message = message
        self._records_effected = records_effected
        self._timestamp = datetime.datetime.utcnow() if timestamp is None else timestamp

    @staticmethod
    def create(job_run, status, message, records_effected, timestamp):
        """Static method for obtaining JobRunLog's DataSet Class
         used for getting the required infomation for Pantomath
        :param job_run: Job Run to be logged
        :type job_run: JobRun
        :param status: Status of the Job Run
        :type status: JobRunLogStatuses
        :param message: Message for the log, defaults to empty string
        :type message: str
        :param records_effected: Number of records effected, defaults to None
        :type records_effected: number
        :param timestamp: Timestamp of the time the job ran, defaults to current UTC time
        :type timestamp: datetime
        ...
        :return: JobRunLog class object
        :rtype: JobRunLog
        """
        return JobRunLog(job_run, status, message, records_effected, timestamp)

    def get_unique_id(self):
        """Returns the UUID of the log
        ...
        :return: Returns the UUID of the log
        :rtype: str
        """
        return self._unique_id

    def get_job_run(self):
        """Returns the logs JobRun
        ...
        :return: Returns the logs JobRun
        :rtype: JobRun
        """
        return self._job_run

    def get_status(self):
        """Returns the status of the log
        ...
        :return: Returns the status of the log
        :rtype: str
        """
        return self._status

    def get_message(self):
        """Returns the message of the log
        ...
        :return: Returns the message of the log
        :rtype: str
        """
        return self._message if self._message else None

    def get_records_effected(self):
        """Returns the number of records effected in the log
        ...
        :return: Returns the number of records effected in the log
        :rtype: number, None
        """
        return int(self._records_effected) if self._records_effected else None

    def get_timestamp(self):
        """Returns the timestamp of the log
        ...
        :return: Returns the timestamp of the log
        :rtype: datetime
        """
        return self._timestamp.strftime("%Y-%m-%dT%H:%M:%S.%fZ")

    def get_asset_paths(self):
        """Returns the asset paths of the log
        ...
        :return: Returns the asset paths of the log
        :rtype: List[AssetPath]
        """
        return self._job_run.get_asset_path()



from pantomath_sdk import PantomathSDK
from pantomath_sdk import AstroDag
from pantomath_sdk import DataikuDataset

good_job = AstroDag(name="TestJob", host_name="testHost")
mock_data_set = DataikuDataset(
    host_name="host_name",
    project_key="project_key",
    dataset_name="dataset_name",
)
good_job_run_log = {
    "job": good_job,
    "name": "TestJobRunLog",
    "source_data_sets": [mock_data_set],
    "target_data_sets": [mock_data_set],
    "next_run_at": "next_run_at",
}


def test_pantomath_sdk_good():
    sdk = PantomathSDK(
        api_base_url="api_base_url_1",
        api_key="api_key_1",
    )
    assert sdk._api_base_url == "api_base_url_1"
    assert sdk._api_key == "api_key_1"


def test_pantomath_sdk_new_job_run_good():
    sdk = PantomathSDK(
        api_base_url="api_base_url_1",
        api_key="api_key_1",
    )
    test_job_run = sdk.new_job_run(
        job=good_job,
        name="name",
        next_run_at="next_run_at",
        source_data_sets=[mock_data_set],
        target_data_sets=[mock_data_set],
    )
    assert (
        test_job_run._get_normalized_message("Hello")
        == "name running in TestJob: Hello"
    )
    post_job = test_job_run.get_job()
    assert post_job == good_job
    testDataSet = test_job_run.get_source_data_sets()
    assert testDataSet[0] == {
        "objectName": mock_data_set.get_name(),
        "objectType": mock_data_set.get_type(),
        "fullyQualifiedObjectName": mock_data_set.get_fully_qualified_object_name(),
    }
    testDataSet = test_job_run.get_target_data_sets()
    assert testDataSet[0] == {
        "objectName": mock_data_set.get_name(),
        "objectType": mock_data_set.get_type(),
        "fullyQualifiedObjectName": mock_data_set.get_fully_qualified_object_name(),
    }
    assert test_job_run.get_next_run_at() == "next_run_at"



from pantomath_sdk import ADFPipeline, PlatformTypes, AssetPathTypes, Asset


def test_adf_pipeline_good():
    node = ADFPipeline(
        pipeline_id=13465478579,
        name="ADFPipeline Unit Test",
    )
    assert node.get_name() == "ADFPipeline Unit Test"
    assert node.get_type() == "ADF_PIPELINE"
    assert node.get_fully_qualified_object_name() == "13465478579"
    assert node.get_platform_type() == PlatformTypes.ADF.value

    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "ADF", "type": "PLATFORM"}, {"depth": 1, "name": "Azure Data Factory Connection", "type": "CONNECTION"}]'
    )


def test_custom_adf_pipeline_asset_path_good():
    node = ADFPipeline(
        pipeline_id=13465478579,
        name="ADFPipeline Unit Test",
        assets=[
            Asset(name="Organization A", type=AssetPathTypes.ORGANIZATION.value),
            Asset(name="Container A", type=AssetPathTypes.CONTAINER.value),
        ],
    )
    # noqa
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "ADF", "type": "PLATFORM"}, {"depth": 1, "name": "Azure Data Factory Connection", "type": "CONNECTION"}, {"depth": 2, "name": "Organization A", "type": "ORGANIZATION"}, {"depth": 3, "name": "Container A", "type": "CONTAINER"}]'
    )



from pantomath_sdk.model.job.job import Job
from pantomath_sdk.enums.job_type import JobTypes
import pantomath_sdk.util as util
from pantomath_sdk.enums.platform_types import PlatformTypes
from typing import Union, List
from pantomath_sdk.model.asset_path.asset_path import Asset, AssetPath


class TableauExtractRefreshTask(Job):
    """TableauExtractRefreshTask's Job Class used for getting the required infomation for Pantomath
    :param host: Host URL
    :type host: str
    :param site_id: ID of the site
    :type site_id: str
    :param refresh_id: The Refresh ID
    :type refresh_id: str
    :param name: Name of the Refresh
    :type name: str
    :param assets: List of assets
    :type assets: List[Asset], optional
    ...
    """

    def __init__(
        self, host, site_id, refresh_id, name, assets: Union[List[Asset], None] = None
    ):
        """Constructor method"""
        self._name = name
        self._host = host
        self._site_id = site_id
        self._refresh_id = refresh_id
        self._asset_path = AssetPath(
            platform_type=PlatformTypes.TABLEAU.value,
            connection_name="Tableau Connection",
            assets=assets,
        )

    @staticmethod
    def create(
        host, site_id, refresh_id, name, assets: Union[List[Asset], None] = None
    ):
        """Static method for obtaining TableauExtractRefreshTask's DataSet Class
        used for getting the required infomation for Pantomath
        :param host: Host URL
        :type host: str
        :param site_id: ID of the site
        :type site_id: str
        :param refresh_id: The Refresh ID
        :type refresh_id: str
        :param name: Name of the Refresh
        :type name: str
        :param assets: List of assets
        :type assets: List[Asset], optional
        ...
        :return: TableauExtractRefreshTask class object
        :rtype: TableauExtractRefreshTask
        """
        return TableauExtractRefreshTask(host, site_id, refresh_id, name, assets)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return util.sanitize_tableau_string(self._name)

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return JobTypes.TABLEAU_EXTRACT_REFRESH_TASK.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return str(
            util.normalize_host(self._host)
            + "/site/"
            + self._site_id
            + "/task/"
            + self._refresh_id
        ).lower()

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: PlatformTypes
        """
        return PlatformTypes.TABLEAU.value

    def get_asset_path(self):
        """Returns the asset path of the object
        ...
        :return: the asset path of the object
        :rtype: AssetPath
        """
        return self._asset_path



from pantomath_sdk.model.job.job import Job
from pantomath_sdk.enums.job_type import JobTypes
from pantomath_sdk.enums.platform_types import PlatformTypes
from typing import Union, List
from pantomath_sdk.model.asset_path.asset_path import Asset, AssetPath


class SqlProcedure(Job):
    """SqlProcedure's Job Class used for getting the required infomation for Pantomath
    :param host: The Host Url
    :type host: str
    :param port: Port number
    :type port: str
    :param database: Name of the database
    :type database: str
    :param schema: Name of the Procedure's schema
    :type schema: str
    :param name: Name of the Procedure
    :type name: str
    ...
    """

    def __init__(
        self,
        host,
        port,
        database,
        schema,
        name,
        assets: Union[List[Asset], None] = None,
    ):
        """Constructor method"""
        self._name = name
        self._schema = schema
        self._database = database
        self._port = port
        self._host = host
        self._assets = assets
        self._asset_path = AssetPath(
            platform_type=PlatformTypes.SQL_SERVER.value,
            connection_name=host,
            assets=assets,
        )

    @staticmethod
    def create(
        host, port, database, schema, name, assets: Union[List[Asset], None] = None
    ):
        """Static method for obtaining SqlProcedure's DataSet Class
        used for getting the required infomation for Pantomath
        :param host: The Host Url
        :type host: str
        :param port: Port number
        :type port: str
        :param database: Name of the database
        :type database: str
        :param schema: Name of the Procedure's schema
        :type schema: str
        :param name: Name of the Procedure
        :type name: str
        ...
        :return: SqlProcedure class object
        :rtype: SqlProcedure
        """
        return SqlProcedure(host, port, database, schema, name, assets)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return JobTypes.SQL_PROCEDURE.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return str(
            str(self._host)
            + ":"
            + str(self._port)
            + "."
            + str(self._database)
            + "."
            + str(self._schema)
            + "."
            + str(self._name)
        )

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.SQL_SERVER.value

    def get_asset_path(self):
        """Returns the asset path of the object
        ...
        :return: the asset path of the object
        :rtype: AssetPath
        """
        return self._asset_path



from pantomath_sdk import JobRunLogStatuses


def test_get_job_run_log_statuses():
    expected_list = [
        "STARTED",
        "IN_PROGRESS",
        "SUCCEEDED",
        "FAILED",
        "QUEUED",
        "WARNING",
    ]
    statuses = JobRunLogStatuses.get_job_run_log_statuses()
    assert statuses.sort() == expected_list.sort()


def test_is_job_run_log_status():
    assert JobRunLogStatuses.is_job_run_log_status("SUCCEEDED")


def test_not_is_job_run_log_status():
    assert not JobRunLogStatuses.is_job_run_log_status("FOO")



from pantomath_sdk.model.dataset.dataset import DataSet
from pantomath_sdk.enums.dataset_type import DataSetTypes
from pantomath_sdk.enums.platform_types import PlatformTypes
import pantomath_sdk.util as util


class DataikuDataset(DataSet):
    """DataikuDataset's DataSet Class used for getting the required infomation for Pantomath
    :param host_name: the dataiku host
    :type host_name: str
    :param project_key: The project key that contains the dataset
    :type project_key: str
    :param dataset_name: Name of the data set
    :type dataset_name: str
    ...
    """

    def __init__(self, host_name, project_key, dataset_name):
        """Constructor method"""
        self._host_name = host_name
        self._project_key = project_key
        self._dataset_name = dataset_name

    @staticmethod
    def create(host_name, project_key, dataset_name):
        """Static method for obtaining DataikuDataset's DataSet Class
        used for getting the required infomation for Pantomath
        :param host_name: the dataiku host
        :type host_name: str
        :param project_key: The project key that contains the dataset
        :type project_key: str
        :param dataset_name: Name of the data set
        :type dataset_name: str
        ...
        :return: DataikuDataset class object
        :rtype: DataikuDataset
        """
        return DataikuDataset(host_name, project_key, dataset_name)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._dataset_name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return DataSetTypes.DATASET.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return util.sanitize_fully_qualified_object_name(
            util.normalize_host(
                str(
                    str(self._host_name)
                    + "/projects/"
                    + str(self._project_key)
                    + "/datasets/"
                    + str(self._dataset_name)
                )
            )
        )

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.DATAIKU.value



from pantomath_sdk import SqlProcedure, PlatformTypes, AssetPathTypes, Asset


def test_sql_procedure_good():
    node = SqlProcedure(
        name="SqlProcedure Unit Test",
        schema="schema",
        database="database",
        port=42,
        host="host",
    )
    assert node.get_name() == "SqlProcedure Unit Test"
    assert node.get_type() == "SQL_PROCEDURE"
    assert (
        node.get_fully_qualified_object_name()
        == "host:42.database.schema.SqlProcedure Unit Test"
    )
    assert node.get_platform_type() == PlatformTypes.SQL_SERVER.value
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "SQL_SERVER", "type": "PLATFORM"}, {"depth": 1, "name": "host", "type": "CONNECTION"}]'
    )


def test_custom_sql_procedure_asset_path_good():
    node = SqlProcedure(
        name="SqlProcedure Unit Test",
        schema="schema",
        database="database",
        port=42,
        host="host",
        assets=[Asset(name="Factory", type=AssetPathTypes.FACTORY.value)],
    )
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "SQL_SERVER", "type": "PLATFORM"}, {"depth": 1, "name": "host", "type": "CONNECTION"}, {"depth": 2, "name": "Factory", "type": "FACTORY"}]'
    )



from pantomath_sdk import DataSetTypes

expected_list = [
    "SQL_TABLE",
    "SQL_VIEW",
    "SQL_MATERIALIZED_VIEW",
    "S3_BUCKET",
    "TABLEAU_WORKBOOK",
    "TABLEAU_DATASOURCE",
    "POWER_BI_DASHBOARD",
    "POWER_BI_REPORT",
    "POWER_BI_DATASET",
    "POWER_BI_DATAFLOW_ENTITY",
    "ADF_DATA_FLOW",
    "ADF_DATASET",
    "FTP",
]


def test_get_dataset_types():
    actual_statuses = DataSetTypes.get_dataset_types()
    assert actual_statuses.sort() == expected_list.sort()


def test_is_dataset_type():
    assert DataSetTypes.is_dataset_type("S3_BUCKET")


def test_not_is_dataset_type():
    assert not DataSetTypes.is_dataset_type("FOO")



from pantomath_sdk import DBTJobConstructor, PlatformTypes, Asset, AssetPathTypes


def test_dbt_job_good():
    node = DBTJobConstructor(
        account_id=146523457734567,
        host="cloud.getdbt.com",
        name="DBTJobConstructor Unit Test",
        job_id=12453576758,
        project_name="DBTJobConstructor Project",
    )
    assert node.get_name() == "DBTJobConstructor Unit Test"
    assert node.get_type() == "JOB"
    assert (
        node.get_fully_qualified_object_name()
        == "cloud.getdbt.com/#/accounts/146523457734567.dbtjobconstructor project."
        "dbtjobconstructor unit test.12453576758"
    )
    assert node.get_platform_type() == PlatformTypes.DBT.value
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "DBT", "type": "PLATFORM"}, {"depth": 1, "name": "DBTJobConstructor Project", "type": "CONNECTION"}]'
    )


def test_custom_dbt_job_asset_path_good():
    node = DBTJobConstructor(
        account_id=146523457734567,
        name="DBTJobConstructor Unit Test",
        host="cloud.getdbt.com",
        job_id=12453576758,
        project_name="DBTJobConstructor Project",
        assets=[
            Asset("Factory", AssetPathTypes.FACTORY.value),
        ],
    )
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "DBT", "type": "PLATFORM"}, {"depth": 1, "name": "DBTJobConstructor Project", "type": "CONNECTION"}, {"depth": 2, "name": "Factory", "type": "FACTORY"}]'
    )



from aenum import Enum


class DataSetTypes(Enum):
    """This class is an enum of all the DataSets supported by the Pantomath Python SDK."""

    SQL_TABLE = "SQL_TABLE"
    SQL_VIEW = "SQL_VIEW"
    SQL_MATERIALIZED_VIEW = "SQL_MATERIALIZED_VIEW"
    S3_BUCKET = "S3_BUCKET"
    TABLEAU_WORKBOOK = "TABLEAU_WORKBOOK"
    TABLEAU_DATASOURCE = "TABLEAU_DATASOURCE"
    POWER_BI_DASHBOARD = "POWER_BI_DASHBOARD"
    POWER_BI_REPORT = "POWER_BI_REPORT"
    POWER_BI_DATASET = "POWER_BI_DATASET"
    POWER_BI_DATAFLOW_ENTITY = "POWER_BI_DATAFLOW_ENTITY"
    ADF_DATA_FLOW = "ADF_DATA_FLOW"
    ADF_DATASET = "ADF_DATASET"
    FTP = "FTP"
    DATASET = "DATASET"
    MODEL = "MODEL"

    @staticmethod
    def get_dataset_types():
        """Get a list of all the enum values
        ...
        :return: list of all the enum values
        :rtype: list of strings
        """
        return dir(DataSetTypes)[: len(DataSetTypes)]

    @staticmethod
    def is_dataset_type(input):
        """Validates if the unput is part of the enum
        :param input: string to check if part of enum
        :type input: str
        ...
        :return: True if in enum else False
        :rtype: Boolean
        """
        return input in DataSetTypes.get_dataset_types()



from pantomath_sdk import DataikuProject, PlatformTypes
from pantomath_sdk.enums.asset_path import AssetPathTypes
from pantomath_sdk.model.asset_path.asset_path import Asset


def test_dataiku_project_good():
    test_node = DataikuProject(
        host_name="host",
        project_key="96868946864896189",
        project_label="DataikuProject Unit Test",
    )
    assert test_node.get_name() == "DataikuProject Unit Test"
    assert test_node.get_type() == "PROJECT"
    assert (
        test_node.get_fully_qualified_object_name() == "host/projects/96868946864896189"
    )
    assert test_node.get_platform_type() == PlatformTypes.DATAIKU.value

    assert (
        test_node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "DATAIKU", "type": "PLATFORM"}, {"depth": 1, "name": "host", "type": "CONNECTION"}]'
    )


def test_custom_dataiku_project_asset_path_good():
    test_node = DataikuProject(
        host_name="host",
        project_key="96868946864896189",
        project_label="DataikuProject Unit Test",
        assets=[
            Asset("Factory", AssetPathTypes.FACTORY.value),
        ],
    )
    assert (
        test_node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "DATAIKU", "type": "PLATFORM"}, {"depth": 1, "name": "host", "type": "CONNECTION"}, {"depth": 2, "name": "Factory", "type": "FACTORY"}]'
    )



import re

try:
    # Python 3
    from urllib.parse import quote
except ImportError:
    # Python 2
    from urllib import quote


def normalize_host(host):
    host = host[:-1] if host[-1:] == "/" else host
    return re.sub(r"/(^\w+:|^)\/\//", "", host)


def sanitize_tableau_string(str_data):
    str_data = re.sub(r"&quot;", "", str_data)
    return re.sub(r'/[\'"]+/g', "", str_data)


def sanitize_string(fqdn):
    return quote(fqdn)


def sanitize_fully_qualified_object_name(fqn):
    return str(sanitize_string(fqn if isinstance(fqn, str) else "")).lower()



from pantomath_sdk.model.job.adf_activity import *
from pantomath_sdk.model.job.adf_pipelines import *
from pantomath_sdk.model.job.astro_dag import *
from pantomath_sdk.model.job.astro_task import *
from pantomath_sdk.model.job.aws_lambda import *
from pantomath_sdk.model.job.dataiku_project import *
from pantomath_sdk.model.job.dataiku_recipe import *
from pantomath_sdk.model.job.dbt_jobs import *
from pantomath_sdk.model.job.dbt_model import *
from pantomath_sdk.model.job.fivetran_connector import *
from pantomath_sdk.model.job.job import *
from pantomath_sdk.model.job.power_bi_data_flow import *
from pantomath_sdk.model.job.power_bi_refresh import *
from pantomath_sdk.model.job.snowflake_pipe import *
from pantomath_sdk.model.job.sql_function import *
from pantomath_sdk.model.job.sql_procedure import *
from pantomath_sdk.model.job.ssis_component import *
from pantomath_sdk.model.job.ssis_package import *
from pantomath_sdk.model.job.ssis_task import *
from pantomath_sdk.model.job.synapse_activity import *
from pantomath_sdk.model.job.synapse_pipeline import *
from pantomath_sdk.model.job.tableau_extract_refresh_task import *
from pantomath_sdk.model.job.control_m_job import *
from pantomath_sdk.model.job.control_m_folder import *



from pantomath_sdk.model.dataset.adf_data_flow import *
from pantomath_sdk.model.dataset.adf_dataset import *
from pantomath_sdk.model.dataset.dataiku_dataset import *
from pantomath_sdk.model.dataset.dataiku_model import *
from pantomath_sdk.model.dataset.dataset import *
from pantomath_sdk.model.dataset.ftp import *
from pantomath_sdk.model.dataset.power_bi_dashboard import *
from pantomath_sdk.model.dataset.power_bi_data_flow_entity import *
from pantomath_sdk.model.dataset.power_bi_dataset import *
from pantomath_sdk.model.dataset.power_bi_report import *
from pantomath_sdk.model.dataset.s3_bucket import *
from pantomath_sdk.model.dataset.sql_materialized_view import *
from pantomath_sdk.model.dataset.sql_table import *
from pantomath_sdk.model.dataset.sql_view import *
from pantomath_sdk.model.dataset.tableau_datasource import *
from pantomath_sdk.model.dataset.tableau_workbook import *



from pantomath_sdk.model.job_run.job_run import JobRun
from pantomath_sdk.client.pantomath_api_client import PantomathApiClient


class PantomathSDK:
    """PantomathSDK's PantomathSDK Class used for creating new job runs
    :param api_base_url: Pantomath's api base URL,black
    which defaults to os.environ["PANTOMATH_API_BASE_URL"]
    :type api_base_url: str, optional
    :param api_key: Pantomath's provided API key,
    which defaults to os.environ["PANTOMATH_API_KEY"]
    :type api_key: str, optional
    ...
    """

    def __init__(self, api_base_url, api_key):
        """Constructor method"""
        self._api_base_url = api_base_url
        self._api_key = api_key

    def new_job_run(
        self,
        job,
        name=None,
        next_run_at=None,
        source_data_sets=None,
        target_data_sets=None,
        job_sources=None,
        job_targets=None,
    ):
        """Creates a new Job Run to store all events to be sent to Pantomath
        :param job: Job Run to be tracked
        :type job: Job,
        :param name: Name of the Job Run
        :type name: str, optional
        :param next_run_at: The type of trigger for when the job will run again
        :type next_run_at: JobDidNotRunAtMethods, optional
        :param source_data_sets: Source Dataset for the job
        :type source_data_sets: Dataset, optional
        :param target_data_sets: Target Dataset for the job
        :type target_data_sets: Dataset, optional
        :param job_sources: Jobs that execute the job
        :type job_sources: Job, optional
        :param job_targets: Jobs executed by the job
        :type job_targets: Job, optional
        ...
        :return: The JobRun to be tracked
        :rtype: JobRun
        """
        client = PantomathApiClient(
            api_base_url=self._api_base_url,
            api_key=self._api_key,
        )
        job_run = JobRun(
            job=job,
            name=name,
            next_run_at=next_run_at,
            source_data_sets=source_data_sets,
            target_data_sets=target_data_sets,
            job_sources=job_sources,
            job_targets=job_targets,
            pantomath_api_client=client,
        )
        return job_run



from pantomath_sdk.model.dataset.dataset import DataSet
from pantomath_sdk.enums.dataset_type import DataSetTypes
from pantomath_sdk.enums.platform_types import PlatformTypes


class S3Bucket(DataSet):
    """S3Bucket's DataSet Class used for getting the required infomation for Pantomath
    :param s3_bucket: The name of the s3_bucket
    :type s3_bucket: str
    ...
    """

    def __init__(self, s3_bucket):
        """Constructor method"""
        self._s3_bucket = s3_bucket

    @staticmethod
    def create(s3_bucket):
        """Static method for obtaining S3Bucket's DataSet Class
        used for getting the required infomation for Pantomath
        :param s3_bucket: The name of the s3_bucket
        :type s3_bucket: str
        ...
        :return: S3Bucket class object
        :rtype: S3Bucket
        """
        return S3Bucket(s3_bucket)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._s3_bucket

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return DataSetTypes.S3_BUCKET.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return self._s3_bucket

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.S3.value



from pantomath_sdk import SSISTask, PlatformTypes, AssetPathTypes, Asset


def test_ssis_task_good():
    node = SSISTask(
        executable_name="_executableName",
        parent_executable_name="_parentExecutableName",
        project_name="projectName",
        folder_name="_folderName",
    )
    assert node.get_name() == "_executableName"
    assert node.get_type() == "TASK"
    assert (
        node.get_fully_qualified_object_name()
        == "_foldername.projectname._parentexecutablename._executablename"
    )
    assert node.get_platform_type() == PlatformTypes.SSIS.value

    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "SSIS", "type": "PLATFORM"}, {"depth": 1, "name": "projectName", "type": "CONNECTION"}]'
    )


def test_custom_ssis_task_asset_path_good():
    node = SSISTask(
        executable_name="_executableName",
        parent_executable_name="_parentExecutableName",
        project_name="projectName",
        folder_name="_folderName",
        assets=[Asset(name="Factory", type=AssetPathTypes.FACTORY.value)],
    )
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "SSIS", "type": "PLATFORM"}, {"depth": 1, "name": "projectName", "type": "CONNECTION"}, {"depth": 2, "name": "Factory", "type": "FACTORY"}]'
    )



from aenum import Enum


class DataVolumeObjectTypes(Enum):
    """This class is an enum of all the DataVolumes supported by the Pantomath Python SDK."""

    SQL_TABLE = "SQL_TABLE"

    @staticmethod
    def get_data_volume_object_types():
        """Get a list of all the enum values
        ...
        :return: list of all the enum values
        :rtype: list of strings
        """
        return dir(DataVolumeObjectTypes)[: len(DataVolumeObjectTypes)]

    @staticmethod
    def is_data_volume_object_type(input):
        """Validates if the unput is part of the enum
        :param input: string to check if part of enum
        :type input: str
        ...
        :return: True if in enum else False
        :rtype: Boolean
        """
        return input in DataVolumeObjectTypes.get_data_volume_object_types()



from pantomath_sdk import DataVolumeObjectTypes

expected_list = ["SQL_TABLE"]


def test_get_data_volume_object_types():
    statuses = DataVolumeObjectTypes.get_data_volume_object_types()
    assert statuses.sort() == expected_list.sort()


def test_is_data_volume_object_type():
    assert DataVolumeObjectTypes.is_data_volume_object_type("SQL_TABLE")


def test_not_is_data_volume_object_type():
    assert not DataVolumeObjectTypes.is_data_volume_object_type("FOO")



import os
import requests
import json

PANTOMATH_SDK_POST_JOB_LOGS_PAGE_SIZE = (
    os.environ["PANTOMATH_SDK_POST_JOB_LOGS_PAGE_SIZE"]
    if "PANTOMATH_SDK_POST_JOB_LOGS_PAGE_SIZE" in os.environ
    else "500"
)
PANTOMATH_API_BASE_URL = (
    os.environ["PANTOMATH_API_BASE_URL"]
    if "PANTOMATH_API_BASE_URL" in os.environ
    else "https://api.dev-pantomath.org/v1"
)
PANTOMATH_API_KEY = (
    os.environ["PANTOMATH_API_KEY"] if "PANTOMATH_API_KEY" in os.environ else ""
)


class PantomathApiClient:
    """This class is used to send the logs to Pantomath API.
    :param api_base_url: Pantomath's api base URL,
    which defaults to os.environ["PANTOMATH_API_BASE_URL"]
    :type api_base_url: str, optional
    :param api_key: Pantomath's provided API key,
    which defaults to os.environ["PANTOMATH_API_KEY"]
    :type api_key: str, optional
    :param job_logs_request_page_size: number of logs
    which can be sent in a since request, defaults to 500
    :type job_logs_request_page_size: str, optional
    """

    def __init__(
        self,
        api_base_url=PANTOMATH_API_BASE_URL,
        api_key=PANTOMATH_API_KEY,
        job_logs_request_page_size=PANTOMATH_SDK_POST_JOB_LOGS_PAGE_SIZE,
    ):
        """Constructor method"""
        self._api_base_url = api_base_url
        self._api_key = api_key
        self._job_logs_request_page_size = job_logs_request_page_size

    def post_job_run_logs(self, jobRunLogs):
        """Method used to validate the run logs,
        separate the run logs into chunks, and sending each chunk to Pantomath
        :param jobRunLogs: list of run logs
        :type jobRunLogs: list of JobRun
        """

        if isinstance(jobRunLogs, list):
            job_run_logs = jobRunLogs
            for log in job_run_logs:
                if not self.validate_log_data(log):
                    raise TypeError("Validation of the requested job runs logs failed.")
            chunk_size = (
                int(self._job_logs_request_page_size)
                if int(self._job_logs_request_page_size) < len(job_run_logs)
                else len(job_run_logs)
            )
            while job_run_logs:
                chunk, job_run_logs = (
                    job_run_logs[:chunk_size],
                    job_run_logs[chunk_size:],
                )
                self._send_logs(chunk)

    def _send_logs(self, logs):
        """Method send run logs to Pantomath
        :param logs: list of run logs
        :type logs: list of JobRun
        ...
        :return: The response of the API
        :rtype: Response. None
        """

        if logs:
            url = self._api_base_url + "/job-run-logs"
            headers = {
                "x-api-key": self._api_key,
                "x-sdk-type": "Python",
                "Accept": "application/json",
                "Content-Type": "application/json",
            }
            data = json.dumps({"jobRunLogs": logs})
            return requests.post(url, headers=headers, data=data)
        return None

    def _validate_dataset(self, dataset):
        """Method used to validate the datasets
        :param dataset: dataset to be checked.
        :type dataset: Job, Dataset
        ...
        :return: If the inputed dataset is valid or not valid for sending to the Pantomath API
        :rtype: Boolean
        """
        return (
            isinstance(dataset["objectName"], str)
            and isinstance(dataset["objectType"], str)
            and isinstance(dataset["fullyQualifiedObjectName"], str)
        )

    def validate_log_data(self, log):
        """Method used to validate the run logs
        :param log: single log to validate
        :type log: JobRun
        ...
        :return: If the inputed log is valid or not valid for sending to the Pantomath API
        :rtype: Boolean
        """
        if not log:
            return False
        try:
            valid_required_args = (
                isinstance(log["jobRunId"], str)
                and isinstance(log["objectName"], str)
                and isinstance(log["objectType"], str)
                and isinstance(log["fullyQualifiedObjectName"], str)
                and isinstance(log["status"], str)
                and isinstance(log["message"], str)
                and isinstance(log["isoTimestamp"], str)
            )
            valid_optional_args = (
                ("recordsEffected" in log and isinstance(log["recordsEffected"], int))
                or not ("recordsEffected" in log)
                or log["recordsEffected"] is None
            )
            valid_source_dataset = "sourceDataSets" in log
            if valid_source_dataset:
                for sdataset in log["sourceDataSets"]:
                    valid_source_dataset = (
                        valid_source_dataset and self._validate_dataset(sdataset)
                    )
            else:
                valid_source_dataset = True
            valid_target_dataset = "targetDataSets" in log
            if valid_target_dataset:
                for tdataset in log["targetDataSets"]:
                    valid_target_dataset = (
                        valid_target_dataset and self._validate_dataset(tdataset)
                    )
            else:
                valid_target_dataset = True
            valid_asset_path = (
                isinstance(log["assetPaths"], object) or log["assetPaths"] is None
            )
            return (
                valid_required_args
                and valid_optional_args
                and valid_source_dataset
                and valid_target_dataset
                and valid_asset_path
            )
        except Exception:
            return False



from pantomath_sdk import JobRun
from pantomath_sdk import AstroDag
from pantomath_sdk import PantomathApiClient
from pantomath_sdk import JobRunLogStatuses
from pantomath_sdk import DataikuDataset
from pantomath_sdk import SqlProcedure
import requests_mock

goodApiClient = PantomathApiClient(api_key="1234")
good_job = AstroDag(name="TestJob", host_name="testHost")

mock_data_set = DataikuDataset(
    host_name="host_name",
    project_key="project_key",
    dataset_name="dataset_name",
)
mock_job = SqlProcedure(
    host="host",
    port=1433,
    database="database",
    schema="schema",
    name="name",
)
good_job_run_log = {
    "job": good_job,
    "name": "TestJobRunLog",
    "source_data_sets": [mock_data_set],
    "target_data_sets": [mock_data_set],
    "job_sources": [mock_job],
    "job_targets": [mock_job],
    "next_run_at": "next_run_at",
    "pantomath_api_client": goodApiClient,
}


def test_PostJobRunLog_get_functions():
    post_job_run_log = JobRun(**good_job_run_log)
    assert (
        post_job_run_log._get_normalized_message("Hello")
        == "TestJobRunLog running in TestJob: Hello"
    )
    post_job = post_job_run_log.get_job()
    assert post_job == good_job
    testDataSet = post_job_run_log.get_source_data_sets()
    assert testDataSet[0] == {
        "objectName": mock_data_set.get_name(),
        "objectType": mock_data_set.get_type(),
        "fullyQualifiedObjectName": mock_data_set.get_fully_qualified_object_name(),
    }
    testDataSet = post_job_run_log.get_target_data_sets()
    assert testDataSet[0] == {
        "objectName": mock_data_set.get_name(),
        "objectType": mock_data_set.get_type(),
        "fullyQualifiedObjectName": mock_data_set.get_fully_qualified_object_name(),
    }
    test_job_sources = post_job_run_log._get_api_job_sources()
    assert test_job_sources[0] == {
        "objectName": mock_job.get_name(),
        "objectType": mock_job.get_type(),
        "fullyQualifiedObjectName": mock_job.get_fully_qualified_object_name(),
    }
    test_job_targets = post_job_run_log._get_api_job_targets()
    assert test_job_targets[0] == {
        "objectName": mock_job.get_name(),
        "objectType": mock_job.get_type(),
        "fullyQualifiedObjectName": mock_job.get_fully_qualified_object_name(),
    }
    assert post_job_run_log.get_next_run_at() == "next_run_at"
    assert (
        post_job_run_log.get_job().get_asset_path().__str__()
        == '[{"depth": 0, "name": "ASTRONOMER", "type": "PLATFORM"}, {"depth": 1, "name": "testHost", "type": "CONNECTION"}]'
    )


def test_unordered_parameters_save_correctly():
    post_job_run_log = JobRun(
        job=good_job,
        name="TestJobRunLog",
        source_data_sets=None,
        target_data_sets=[mock_data_set],
        job_targets=[mock_job],
        next_run_at="next_run_at",
        pantomath_api_client=goodApiClient,
    )

    assert post_job_run_log.get_source_data_sets() == []
    assert post_job_run_log.get_target_data_sets() == [
        {
            "objectName": mock_data_set.get_name(),
            "objectType": mock_data_set.get_type(),
            "fullyQualifiedObjectName": mock_data_set.get_fully_qualified_object_name(),
        }
    ]
    assert post_job_run_log.get_job_sources() == []
    assert post_job_run_log.get_job_targets() == [mock_job]


def test_PostJobRunLog_log_start_without_message():
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        post_job_run_log = JobRun(**good_job_run_log)
        post_job_run_log.log_start(records_effected=42)
        assert len(post_job_run_log._log_buffer) == 0
        assert mock.last_request.json().get("jobRunLogs")[0]["recordsEffected"] == 42
        assert (
            mock.last_request.json().get("jobRunLogs")[0]["status"]
            == JobRunLogStatuses.STARTED.value
        )
        assert (
            mock.last_request.json().get("jobRunLogs")[0]["message"]
            == "TestJobRunLog running in TestJob: Started"
        )
        assert mock.call_count == 1


def test_PostJobRunLog_log_start_with_message():
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        post_job_run_log = JobRun(**good_job_run_log)
        post_job_run_log.log_start(records_effected=42, message="Test")
        assert len(post_job_run_log._log_buffer) == 0
        assert mock.last_request.json().get("jobRunLogs")[0]["recordsEffected"] == 42
        assert (
            mock.last_request.json().get("jobRunLogs")[0]["status"]
            == JobRunLogStatuses.STARTED.value
        )
        assert (
            mock.last_request.json().get("jobRunLogs")[0]["message"]
            == "TestJobRunLog running in TestJob: Test"
        )
        assert mock.call_count == 1


def test_PostJobRunLog_log_progress_without_message():
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        post_job_run_log = JobRun(**good_job_run_log)
        post_job_run_log.log_progress(records_effected=42)
        assert len(post_job_run_log._log_buffer) == 1
        assert post_job_run_log._log_buffer[0]["recordsEffected"] == 42
        assert (
            post_job_run_log._log_buffer[0]["status"]
            == JobRunLogStatuses.IN_PROGRESS.value
        )
        assert (
            post_job_run_log._log_buffer[0]["message"]
            == "TestJobRunLog running in TestJob: Progressing"
        )
        assert mock.call_count == 0


def test_PostJobRunLog_log_progress_with_message():
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        post_job_run_log = JobRun(**good_job_run_log)
        post_job_run_log.log_progress(records_effected=42, message="Test")
        assert len(post_job_run_log._log_buffer) == 1
        assert post_job_run_log._log_buffer[0]["recordsEffected"] == 42
        assert (
            post_job_run_log._log_buffer[0]["status"]
            == JobRunLogStatuses.IN_PROGRESS.value
        )
        assert (
            post_job_run_log._log_buffer[0]["message"]
            == "TestJobRunLog running in TestJob: Test"
        )
        assert mock.call_count == 0


def test_PostJobRunLog_log_warning_without_message():
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        post_job_run_log = JobRun(**good_job_run_log)
        post_job_run_log.log_warning(records_effected=42)
        assert len(post_job_run_log._log_buffer) == 1
        assert post_job_run_log._log_buffer[0]["recordsEffected"] == 42
        assert (
            post_job_run_log._log_buffer[0]["status"] == JobRunLogStatuses.WARNING.value
        )
        assert (
            post_job_run_log._log_buffer[0]["message"]
            == "TestJobRunLog running in TestJob: Warning"
        )
        assert mock.call_count == 0


def test_PostJobRunLog_log_warning_with_message():
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        post_job_run_log = JobRun(**good_job_run_log)
        post_job_run_log.log_warning(records_effected=42, message="Test")
        assert len(post_job_run_log._log_buffer) == 1
        assert post_job_run_log._log_buffer[0]["recordsEffected"] == 42
        assert (
            post_job_run_log._log_buffer[0]["status"] == JobRunLogStatuses.WARNING.value
        )
        assert (
            post_job_run_log._log_buffer[0]["message"]
            == "TestJobRunLog running in TestJob: Test"
        )
        assert mock.call_count == 0


def test_PostJobRunLog_log_success_without_message():
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        post_job_run_log = JobRun(**good_job_run_log)
        post_job_run_log.log_success(records_effected=42)
        assert len(post_job_run_log._log_buffer) == 0
        assert mock.call_count == 1


def test_PostJobRunLog_log_success_with_message():
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        post_job_run_log = JobRun(**good_job_run_log)
        post_job_run_log.log_success(records_effected=42, message="Test")
        assert len(post_job_run_log._log_buffer) == 0
        assert mock.call_count == 1


def test_PostJobRunLog_log_failed_without_message():
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        post_job_run_log = JobRun(**good_job_run_log)
        post_job_run_log.log_failure(records_effected=42)
        assert len(post_job_run_log._log_buffer) == 0
        assert mock.call_count == 1


def test_PostJobRunLog_log_failed_with_message():
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        post_job_run_log = JobRun(**good_job_run_log)
        post_job_run_log.log_failure(records_effected=42, message="Test")
        assert len(post_job_run_log._log_buffer) == 0
        assert mock.call_count == 1



from pantomath_sdk.model.job.job import Job
from pantomath_sdk.enums.job_type import JobTypes
from pantomath_sdk.enums.platform_types import PlatformTypes
from typing import Union, List
from pantomath_sdk.model.asset_path.asset_path import Asset, AssetPath


class SSISPackage(Job):
    """SSISPackage's Job Class used for getting the required infomation for Pantomath
    :param project_name: Name of the Project
    :type project_name: str
    :param folder_name: Name of the Folder
    :type folder_name: str
    :param name: Name of the Package
    :type name: str
    ...
    """

    def __init__(
        self, project_name, folder_name, name, assets: Union[List[Asset], None] = None
    ):
        """Constructor method"""
        self._name = name
        self._folder_name = folder_name
        self._project_name = project_name
        self._assets = assets
        self._asset_path = AssetPath(
            platform_type=PlatformTypes.SSIS.value,
            connection_name=project_name,
            assets=assets,
        )

    @staticmethod
    def create(
        project_name, folder_name, name, assets: Union[List[Asset], None] = None
    ):
        """Static method for obtaining SSISPackage's DataSet Class
        used for getting the required infomation for Pantomath
        :param project_name: Name of the Project
        :type project_name: str
        :param folder_name: Name of the Folder
        :type folder_name: str
        :param name: Name of the Package
        :type name: str
        ...
        :return: SSISPackage class object
        :rtype: SSISPackage
        """
        return SSISPackage(project_name, folder_name, name, assets)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return JobTypes.PACKAGE.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return str(
            self._folder_name + "." + self._project_name + "." + self._name
        ).lower()

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.SSIS.value

    def get_asset_path(self):
        """Returns the asset path of the object
        ...
        :return: the asset path of the object
        :rtype: AssetPath
        """
        return self._asset_path



from pantomath_sdk import PowerBIDashboard, PlatformTypes


def test_powerbi_dashboard_good():
    test_node = PowerBIDashboard(
        workspace_id=123124124,
        dashboard_id=9879789789,
        name="powerbi_dashboard Unit Test",
    )
    assert test_node.get_name() == "powerbi_dashboard Unit Test"
    assert test_node.get_type() == "POWER_BI_DASHBOARD"
    assert (
        test_node.get_fully_qualified_object_name()
        == "app.powerbi.com/groups/123124124/dashboards/9879789789"
    )
    assert test_node.get_platform_type() == PlatformTypes.POWERBI.value



from pantomath_sdk.enums.job_run_log_status import JobRunLogStatuses
from pantomath_sdk.model.job_run.job_run_log import JobRunLog
from pantomath_sdk.model.job.job import Job
import datetime
import uuid


class JobRun(Job):
    """JobRun's Job Class used for getting the required information for Pantomath
    :param pantomath_api_client: The Pantomath API Client Object
    :type pantomath_api_client: PantomathApiClient
    :param job: Job which will be tracked
    :type job: Job
    :param name: Name of the Job Run
    :type name: str
    :param next_run_at: The type of trigger for when the job will run again
    :type next_run_at: JobDidNotRunAtMethods
    :param source_data_sets: Source Dataset for the job
    :type source_data_sets: Dataset
    :param target_data_sets: Target Dataset for the job
    :type target_data_sets: Dataset
    :param job_sources: Jobs that execute the job
    :type job_sources: Job
    :param job_targets: Jobs executed by the job
    :type job_targets: Job
    ...
    """

    def __init__(
        self,
        pantomath_api_client,
        job,
        name=None,
        next_run_at=None,
        source_data_sets=None,
        target_data_sets=None,
        job_sources=None,
        job_targets=None,
    ):
        """Constructor method"""
        self._job = job
        self._id = uuid.uuid4()
        self._name = name
        self._source_data_sets = (
            source_data_sets if source_data_sets is not None else []
        )
        self._target_data_sets = (
            target_data_sets if target_data_sets is not None else []
        )
        self._job_sources = job_sources if job_sources is not None else []
        self._job_targets = job_targets if job_targets is not None else []
        self._next_run_at = next_run_at
        self._pantomath_api_client = pantomath_api_client
        self._log_buffer = []
        self._pushing_logs = False
        self._reported_start = False
        self._reported_success = False
        self._reported_failure = False

    @staticmethod
    def create(
        pantomath_api_client,
        job,
        name,
        next_run_at,
        source_data_sets,
        target_data_sets,
        job_sources,
        job_targets,
    ):
        """Static method for obtaining JobRun's DataSet Class
        used for getting the required infomation for Pantomath
        :param pantomath_api_client: The Pantomath API Client Object
        :type pantomath_api_client: PantomathApiClient
        :param job: Job which will be tracked
        :type job: Job
        :param name: Name of the Job Run
        :type name: str
        :param next_run_at: The type of trigger for when the job will run again
        :type next_run_at: JobDidNotRunAtMethods
        :param source_data_sets: Source Dataset for the job
        :type source_data_sets: Dataset
        :param target_data_sets: Target Dataset for the job
        :type target_data_sets: Dataset
        :param job_sources: Jobs that execute the job
        :type job_sources: Job
        :param job_targets: Jobs executed by the job
        :type job_targets: Job
        ...
        :return: JobRun class object
        :rtype: JobRun
        """
        return JobRun(
            pantomath_api_client,
            job,
            name,
            next_run_at,
            source_data_sets,
            target_data_sets,
            job_sources,
            job_targets,
        )

    def _get_normalized_message(self, message):
        """Processed the message to be into a normalized format for Pantomath
        :param message: message for the run
        :type message: str
        ...
        :return: Processed message
        :rtype: str
        """
        lengthCappedMessage = message[:1000]
        jobRunNamePrefix = (self._name + " running in ") if self._name else ""
        jobName = self._job.get_name()
        rtnMsg = jobRunNamePrefix + jobName + ": " + lengthCappedMessage
        return rtnMsg

    def get_job(self):
        """Returns the Job
        ...
        :return: Returns the Job
        :rtype: Job
        """
        return self._job

    def get_job_run_id(self):
        """Returns the Job run id
        ...
        :return: Returns the Job run id
        :rtype: str
        """
        return self._id

    def get_source_data_sets(self):
        """Returns source datasets
        ...
        :return: Returns source datasets
        :rtype: Dataset
        """
        source_array = []
        for dataset in self._source_data_sets:
            source_array.append(
                {
                    "objectName": dataset.get_name(),
                    "objectType": dataset.get_type(),
                    "fullyQualifiedObjectName": dataset.get_fully_qualified_object_name(),
                }
            )
        return source_array

    def get_target_data_sets(self):
        """Returns target datasets
        ...
        :return: Returns target datasets
        :rtype: Dataset
        """
        target_array = []
        for dataset in self._target_data_sets:
            target_array.append(
                {
                    "objectName": dataset.get_name(),
                    "objectType": dataset.get_type(),
                    "fullyQualifiedObjectName": dataset.get_fully_qualified_object_name(),
                }
            )
        return target_array

    def get_job_sources(self):
        """Returns job sources
        ...
        :return: Returns job sources
        :rtype: Job
        """
        return self._job_sources

    def get_job_targets(self):
        """Returns job targets
        ...
        :return: Returns job targets
        :rtype: Job
        """
        return self._job_targets

    def get_next_run_at(self):
        """Returns Next Run At Type
        ...
        :return: Returns Next Run At Type
        :rtype: JobDidNotRunAtMethods
        """
        return self._next_run_at

    def log_start(self, message="Started", records_effected=None):
        """Create Log for the Job start event
        :param message: Message to attach to the log
        :type message: str
        :param records_effected: Number of record effected by the job, defaults to none
        :type records_effected: Number, None
        ...
        """
        if self._reported_start:
            raise Exception(
                "report_Start can only be called once per JobRun instance. Job Run "
                + self._job["name"]
            )
        self._reported_start = True
        job_run_log = JobRunLog(
            job_run=self,
            status=JobRunLogStatuses.STARTED.value,
            message=self._get_normalized_message(message),
            records_effected=records_effected,
            timestamp=datetime.datetime.utcnow(),
        )
        self._convert_to_api_log(job_run_log)
        self.log_progress(message="Start In Progress Log")
        self._push_and_clear_log_buffer()

    def log_progress(self, message="Progressing", records_effected=None):
        """Create Log for the Job Progressing event
        :param message: Message to attach to the log
        :type message: str
        :param records_effected: Number of record effected by the job, defaults to None
        :type records_effected: Number, None
        ...
        """
        job_run_log = JobRunLog(
            job_run=self,
            status=JobRunLogStatuses.IN_PROGRESS.value,
            message=self._get_normalized_message(message),
            records_effected=records_effected,
            timestamp=datetime.datetime.utcnow(),
        )
        self._convert_to_api_log(job_run_log)

    def log_warning(self, message="Warning", records_effected=None):
        """Create Log for the Job Warning event
        :param message: Message to attach to the log
        :type message: str
        :param records_effected: Number of record effected by the job, defaults to none
        :type records_effected: Number, None
        ...
        """
        job_run_log = JobRunLog(
            job_run=self,
            status=JobRunLogStatuses.WARNING.value,
            message=self._get_normalized_message(message),
            records_effected=records_effected,
            timestamp=datetime.datetime.utcnow(),
        )
        self._convert_to_api_log(job_run_log)

    def log_success(self, message="Succeeded", records_effected=None):
        """Create Log for the Job Succeeded event
        :param message: Message to attach to the log
        :type message: str
        :param records_effected: Number of record effected by the job, defaults to none
        :type records_effected: Number, None
        ...
        """
        if self._reported_success:
            raise Exception(
                "reportSuccess can only be called once per JobRun instance. Job Run: "
                + self._job["name"]
            )
        if self._reported_failure:
            raise Exception(
                "reportSuccess cannot be called because reportFailure has already been"
                "called. Job Run: " + self._job["name"]
            )
        self._reported_success = True
        job_run_log = JobRunLog(
            job_run=self,
            status=JobRunLogStatuses.SUCCEEDED.value,
            message=self._get_normalized_message(message),
            records_effected=records_effected,
            timestamp=datetime.datetime.utcnow(),
        )
        self._convert_to_api_log(job_run_log)
        self._push_and_clear_log_buffer()

    def log_failure(self, message="Failed", records_effected=None):
        """Create Log for the Job Failed event
        :param message: Message to attach to the log
        :type message: str
        :param records_effected: Number of record effected by the job, defaults to none
        :type records_effected: Number, None
        ...
        """
        if self._reported_success:
            raise Exception(
                "reportFailure can only be called once per JobRun instance. Job Run: "
                + self._job["name"]
            )
        if self._reported_failure:
            raise Exception(
                "reportFailure cannot be called because reportSuccess has already been"
                "called. Job Run: " + self._job["name"]
            )
        self._reported_success = True
        job_run_log = JobRunLog(
            job_run=self,
            status=JobRunLogStatuses.FAILED.value,
            message=self._get_normalized_message(message),
            records_effected=records_effected,
            timestamp=datetime.datetime.utcnow(),
        )
        self._convert_to_api_log(job_run_log)
        self._push_and_clear_log_buffer()

    def _get_api_job_sources(self):
        """Returns job sources
        ...
        :return: Returns job sources
        :rtype:
        """
        source_array = []
        for job in self._job_sources:
            source_array.append(
                {
                    "objectName": job.get_name(),
                    "objectType": job.get_type(),
                    "fullyQualifiedObjectName": job.get_fully_qualified_object_name(),
                }
            )
        return source_array

    def _get_api_job_targets(self):
        """Returns job targets
        ...
        :return: Returns job targets
        :rtype:
        """
        target_array = []
        for job in self._job_targets:
            target_array.append(
                {
                    "objectName": job.get_name(),
                    "objectType": job.get_type(),
                    "fullyQualifiedObjectName": job.get_fully_qualified_object_name(),
                }
            )
        return target_array

    def _convert_to_api_log(self, job_run_log):
        """Convert the Job Run Log into an API ready Object and adds to the buffer
        :param job_run_log: Message to attach to the log
        :type job_run_log: JobRunLog
        ...
        """
        api_log = {
            "uniqueId": str(uuid.uuid4()),
            "jobRunId": str(self.get_job_run_id()),
            "objectName": str(self._job.get_name()),
            "objectType": str(self._job.get_type()),
            "fullyQualifiedObjectName": str(
                self._job.get_fully_qualified_object_name()
            ),
            "status": job_run_log.get_status(),
            "message": str(job_run_log.get_message()),
            "targetDataSets": self.get_target_data_sets(),
            "sourceDataSets": self.get_source_data_sets(),
            "jobSources": self._get_api_job_sources(),
            "jobTargets": self._get_api_job_targets(),
            "isoTimestamp": str(job_run_log.get_timestamp()),
            "assetPaths": [{"assets": self._job.get_asset_path().get_asset_path()}],
        }
        if job_run_log.get_records_effected():
            api_log["recordsEffected"] = int(job_run_log.get_records_effected())
        self._log_buffer.append(api_log)

    def _push_and_clear_log_buffer(self):
        """Sends the buffer to the Pantomath API
        ...
        """
        if not self._pushing_logs and len(self._log_buffer) > 0:
            self._pushing_logs = True
            try:
                self._pantomath_api_client.post_job_run_logs(
                    jobRunLogs=self._log_buffer
                )
            except Exception as e:
                print(e)
            finally:
                self._pushing_logs = False
            self._log_buffer = []



from pantomath_sdk import JobTypes


def test_get_job_types():
    expected_list = [
        "DAG",
        "TASK",
        "SQL_FUNCTION",
        "SQL_PROCEDURE",
        "TABLEAU_EXTRACT_REFRESH_TASK",
        "FIVETRAN_CONNECTOR",
        "DBT_MODEL",
        "POWER_BI_REFRESH",
        "POWER_BI_DATAFLOW",
        "ADF_ACTIVITY",
        "ADF_PIPELINE",
        "JOB",
        "ACTIVITY",
        "PIPELINE",
        "PACKAGE",
        "COMPONENT",
        "TASK",
        "SNOWFLAKE_PIPE",
        "AWS_LAMBDA",
    ]
    statuses = JobTypes.get_job_types()
    assert statuses.sort() == expected_list.sort()


def test_is_job_type():
    assert JobTypes.is_job_type("PIPELINE")


def test_not_is_job_type():
    assert not JobTypes.is_job_type("FOO")



from pantomath_sdk import ADFDataset, PlatformTypes


def test_adf_dataset_good():
    test_node = ADFDataset(data_set_id="96868946864896189", name="ADF Unit Test")
    assert test_node.get_name() == "ADF Unit Test"
    assert test_node.get_type() == "ADF_DATASET"
    assert test_node.get_fully_qualified_object_name() == "96868946864896189"
    assert test_node.get_platform_type() == PlatformTypes.ADF.value



import unittest
from pantomath_sdk import AssetPath, Asset, AssetPathTypes


class TestAssetPath(unittest.TestCase):
    def setUp(self):
        self.platform_type = "TestPlatform"
        self.connection_name = "TestConnection"
        self.account = "TestAccount"
        self.agent = "TestAgent"
        self.asset_path = AssetPath(
            self.platform_type,
            self.connection_name,
            assets=[
                Asset(self.account, AssetPathTypes.ACCOUNT.value),
                Asset(self.agent, AssetPathTypes.AGENT.value),
            ],
        )

    def test_initial_assets(self):
        self.assertEqual(self.asset_path.assets[0].name, self.platform_type)
        self.assertEqual(self.asset_path.assets[0].type, AssetPathTypes.PLATFORM.value)
        self.assertEqual(self.asset_path.assets[1].name, self.connection_name)
        self.assertEqual(
            self.asset_path.assets[1].type, AssetPathTypes.CONNECTION.value
        )

        # Test user defined assets
        self.assertEqual(self.asset_path.assets[2].name, self.account)
        self.assertEqual(self.asset_path.assets[2].type, AssetPathTypes.ACCOUNT.value)
        self.assertEqual(self.asset_path.assets[3].name, self.agent)
        self.assertEqual(self.asset_path.assets[3].type, AssetPathTypes.AGENT.value)

    def test_get_path(self):
        expected_path = (
            f"{self.platform_type}/{self.connection_name}/{self.account}/{self.agent}"
        )
        self.assertEqual(self.asset_path.get_path, expected_path)

    def test_get_asset_path(self):
        assets = self.asset_path.get_asset_path()
        self.assertIsInstance(assets, object)
        self.assertEqual(len(assets), 4)

    def test_str(self):
        # noqa: For testing purposes
        expected_str = '[{"depth": 0, "name": "TestPlatform", "type": "PLATFORM"}, {"depth": 1, "name": "TestConnection", "type": "CONNECTION"}, {"depth": 2, "name": "TestAccount", "type": "ACCOUNT"}, {"depth": 3, "name": "TestAgent", "type": "AGENT"}]'  # noqa
        self.assertEqual(str(self.asset_path), expected_str)

    def test_append_asset(self):
        new_asset_name = "TestContainer"
        new_asset_type = AssetPathTypes.CONTAINER.value
        self.asset_path.append_asset(new_asset_name, new_asset_type)
        self.assertEqual(len(self.asset_path.assets), 5)
        self.assertEqual(self.asset_path.assets[4].name, new_asset_name)
        self.assertEqual(self.asset_path.assets[4].type, new_asset_type)



class DataSet(object):
    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        raise NotImplementedError()

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        raise NotImplementedError()

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        raise NotImplementedError()

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        raise NotImplementedError()



from pantomath_sdk import DataikuDataset, PlatformTypes


def test_dataiku_dataset_good():
    test_node = DataikuDataset(
        host_name="host",
        project_key="96868946864896189",
        dataset_name="DataikuDataset Unit Test",
    )
    assert test_node.get_name() == "DataikuDataset Unit Test"
    assert test_node.get_type() == "DATASET"
    assert (
        test_node.get_fully_qualified_object_name()
        == "host/projects/96868946864896189/datasets/dataikudataset%20unit%20test"
    )
    assert test_node.get_platform_type() == PlatformTypes.DATAIKU.value



class PostJobRunLog:
    """PostJobRunLog's PostJobRunLog Class used for validating Logs before sending to Pantomath
    :param job_run_id: ID of the Job Run
    :type job_run_id: PantomathApiClient
    :param object_name: Name of the Job Run Log
    :type object_name: str
    :param object_type: Status of the Job Run Log
    :type object_type: JobRunLogStatuses
    :param fully_qualified_object_name: Fully Qualified Name for the Job Run
    :type fully_qualified_object_name: str
    :param source_data_sets: Source Dataset for the job
    :type source_data_sets: Dataset
    :param target_data_sets: Target Dataset for the job
    :type target_data_sets: Dataset
    :param iso_timestamp: Timestamp of the log
    :type iso_timestamp: Datetime
    ...
    """

    def __init__(
        self,
        job_run_id,
        object_name,
        object_type,
        asset_path,
        fully_qualified_object_name,
        status,
        message,
        records_effected,
        source_data_sets,
        target_data_sets,
        iso_timestamp,
    ):
        """Constructor method"""
        self.job_run_id = job_run_id
        self.object_name = object_name
        self.object_type = object_type
        self.fully_qualified_object_name = fully_qualified_object_name
        self.asset_path = asset_path
        self.status = status
        self.message = message
        self.records_effected = records_effected
        self.source_data_sets = source_data_sets
        self.target_data_sets = target_data_sets
        self.iso_timestamp = iso_timestamp
        if not self.validate_data():
            raise TypeError("Validation of the requestion job runs logs failed.")

    @staticmethod
    def create(
        job_run_id,
        object_name,
        object_type,
        fully_qualified_object_name,
        asset_path,
        status,
        message,
        records_effected,
        source_data_sets,
        target_data_sets,
        iso_timestamp,
    ):
        """Static method for obtaining JobRun's DataSet Class
        used for getting the required infomation for Pantomath
        :param job_run_id: ID of the Job Run
        :type job_run_id: PantomathApiClient
        :param object_name: Name of the Job Run Log
        :type object_name: str
        :param object_type: Status of the Job Run Log
        :type object_type: JobRunLogStatuses
        :param fully_qualified_object_name: Fully Qualified Name for the Job Run
        :type fully_qualified_object_name: str
        :param source_data_sets: Source Dataset for the job
        :type source_data_sets: Dataset
        :param target_data_sets: Target Dataset for the job
        :type target_data_sets: Dataset
        :param iso_timestamp: Timestamp of the log
        :type iso_timestamp: Datetime
        ...
        :return: PostJobRunLog class object
        :rtype: PostJobRunLog
        """
        return PostJobRunLog(
            job_run_id,
            object_name,
            object_type,
            fully_qualified_object_name,
            asset_path,
            status,
            message,
            records_effected,
            source_data_sets,
            target_data_sets,
            iso_timestamp,
        )

    def _validate_dataset(self, dataset):
        """Validates the dataset
        :param dataset: Dataset to be validated
        :type dataset: Dataset
        ...
        :return: Return if the dataset is valid
        :rtype: Boolean
        """
        return (
            "object_name" in dataset
            and isinstance(dataset["object_name"], str)
            and "object_type" in dataset
            and isinstance(dataset["object_type"], str)
            and "fully_qualified_object_name" in dataset
            and isinstance(dataset["fully_qualified_object_name"], str)
        )

    def validate_data(self):
        """Validates the PostJobRunLog
        ...
        :return: Return if the PostJobRunLog is valid
        :rtype: Boolean
        """
        valid_required_args = (
            isinstance(self.job_run_id, str)
            and isinstance(self.object_name, str)
            and isinstance(self.object_type, str)
            and isinstance(self.fully_qualified_object_name, str)
            and isinstance(self.status, str)
            and isinstance(self.message, str)
            and isinstance(self.iso_timestamp, str)
        )
        valid_optional_args = isinstance(self.records_effected, int) or isinstance(
            self.records_effected, None
        )

        valid_asset_path = isinstance(self.asset_path, list) or isinstance(
            self.asset_path, None
        )
        valid_source_dataset = True
        if self.source_data_sets:
            for sdataset in self.source_data_sets:
                valid_source_dataset = valid_source_dataset and self._validate_dataset(
                    sdataset
                )
        valid_target_dataset = True
        if self.target_data_sets:
            for tdataset in self.target_data_sets:
                valid_target_dataset = valid_target_dataset and self._validate_dataset(
                    tdataset
                )
        return (
            valid_required_args
            and valid_optional_args
            and valid_source_dataset
            and valid_target_dataset
            and valid_asset_path
        )



from typing import Union, List
from pantomath_sdk.enums.platform_types import PlatformTypes
from pantomath_sdk.model.asset_path.asset_path import Asset, AssetPath
from pantomath_sdk.model.job.job import Job
from pantomath_sdk.enums.job_type import JobTypes
import pantomath_sdk.util as util


class AstroDag(Job):
    """AstroDag's Job Class used for getting the required infomation for Pantomath
    :param host_name: NAme of the host
    :type host_name: str
    :param name: Name of the DAG
    :type name: str
    :param assets: List of assets
    :type assets: List[Asset]
    ...
    """

    def __init__(self, host_name, name, assets: Union[List[Asset], None] = None):
        """Constructor method"""
        self._host_name = host_name
        self._name = name
        self._asset_path = AssetPath(
            platform_type=PlatformTypes.ASTRONOMER.value,
            connection_name=self._host_name,
            assets=assets,
        )  # Default value

    @staticmethod
    def create(host_name, name, assets: Union[List[Asset], None] = None):
        """Static method for obtaining AstroDag's DataSet Class
        used for getting the required infomation for Pantomath
        :param host_name: NAme of the host
        :type host_name: str
        :param name: Name of the DAG
        :type name: str
        :param assets: List of assets
        :type assets: List[Asset]
        ...
        :return: AstroDag class object
        :rtype: AstroDag
        """
        return AstroDag(host_name, name, assets)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return JobTypes.DAG.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return util.sanitize_fully_qualified_object_name(
            self._host_name + "/" + self.get_name()
        )

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.ASTRO.value

    def get_asset_path(self) -> AssetPath:
        """Returns the asset paths of the object
        ...
        :return: the asset paths of the object
        :rtype: AssetPath
        """
        return self._asset_path



from pantomath_sdk.model.job.job import Job
from pantomath_sdk.enums.job_type import JobTypes
from pantomath_sdk.enums.platform_types import PlatformTypes
from typing import Union, List
from pantomath_sdk.model.asset_path.asset_path import Asset, AssetPath


class SSISComponent(Job):
    """SSISComponent's Job Class used for getting the required infomation for Pantomath
    :param project_name: Name of the project
    :type project_name: str
    :param execution_path: Path of Execution
    :type execution_path: str
    :param folder_name: Name of the folder
    :type folder_name: str
    :param name: Name of the component
    :type name: str
    ...
    """

    def __init__(
        self,
        project_name,
        execution_path,
        folder_name,
        name,
        assets: Union[List[Asset], None] = None,
    ):
        """Constructor method"""
        self._name = name
        self._folder_name = folder_name
        self._execution_path = execution_path
        self._project_name = project_name
        self._asset_path = AssetPath(
            platform_type=PlatformTypes.SSIS.value,
            connection_name=project_name,
            assets=assets,
        )

    @staticmethod
    def create(
        project_name,
        execution_path,
        folder_name,
        name,
        assets: Union[List[Asset], None] = None,
    ):
        """Static method for obtaining SSISComponent's DataSet Class
        used for getting the required infomation for Pantomath
        :param project_name: Name of the project
        :type project_name: str
        :param execution_path: Path of Execution
        :type execution_path: str
        :param folder_name: Name of the folder
        :type folder_name: str
        :param name: Name of the component
        :type name: str
        ...
        :return: SSISComponent class object
        :rtype: SSISComponent
        """
        return SSISComponent(project_name, execution_path, folder_name, name, assets)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return JobTypes.COMPONENT.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return str(
            self._folder_name
            + "."
            + self._project_name
            + "."
            + self._execution_path
            + "."
            + self._name
        ).lower()

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.SSIS.value

    def get_asset_path(self):
        """Returns the asset path of the object
        ...
        :return: the asset path of the object
        :rtype: AssetPath
        """
        return self._asset_path



from pantomath_sdk import ControlMJob, PlatformTypes


def test_control_m_folder_good():
    node = ControlMJob(
        host="mockHost",
        server="mockserver",
        folder_hierarchy=["A", "B", "C"],
        name="ControlMJob Unit Test",
    )
    assert node.get_name() == "ControlMJob Unit Test"
    assert node.get_type() == "JOB"
    assert (
        node.get_fully_qualified_object_name()
        == "mockhost.mockserver.a.b.c.controlmjob%20unit%20test"
    )
    assert node.get_platform_type() == PlatformTypes.CONTROL_M.value
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "CONTROL_M", "type": "PLATFORM"}, {"depth": 1, "name": "mockHost", "type": "CONNECTION"}, {"depth": 2, "name": "mockserver", "type": "SERVER"}, {"depth": 3, "name": "A", "type": "FOLDER"}, {"depth": 4, "name": "B", "type": "FOLDER"}, {"depth": 5, "name": "C", "type": "FOLDER"}]'
    )


def test_control_m_folder_good_no_optional():
    node = ControlMJob(
        host="mockHost",
        server="mockserver",
        name="ControlMJob Unit Test",
    )
    assert node.get_name() == "ControlMJob Unit Test"
    assert node.get_type() == "JOB"
    assert (
        node.get_fully_qualified_object_name()
        == "mockhost.mockserver.controlmjob%20unit%20test"
    )
    assert node.get_platform_type() == PlatformTypes.CONTROL_M.value
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "CONTROL_M", "type": "PLATFORM"}, {"depth": 1, "name": "mockHost", "type": "CONNECTION"}, {"depth": 2, "name": "mockserver", "type": "SERVER"}]'
    )



from pantomath_sdk.model.dataset.dataset import DataSet
from pantomath_sdk.enums.dataset_type import DataSetTypes
import pantomath_sdk.util as util
from pantomath_sdk.enums.platform_types import PlatformTypes


class ADFDataset(DataSet):
    """ADFDataset's DataSet Class used for getting the required infomation for Pantomath
    :param data_set_id: The ID of the data set
    :type data_set_id: str
    :param name: Name of the data set
    :type name: str
    ...
    """

    def __init__(self, data_set_id, name):
        """Constructor method"""
        self._data_set_id = data_set_id
        self._name = name

    @staticmethod
    def create(data_set_id, name):
        """Static method for obtaining ADFDataset's DataSet Class
        used for getting the required infomation for Pantomath
        :param data_set_id: The ID of the data set
        :type data_set_id: str
        :param name: Name of the data set
        :type name: str
        ...
        :return: ADFDataset class object
        :rtype: ADFDataset
        """
        return ADFDataset(data_set_id, name)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return DataSetTypes.ADF_DATASET.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return util.sanitize_fully_qualified_object_name(self._data_set_id)

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.ADF.value



from typing import Union, List
from pantomath_sdk.model.asset_path.asset_path import Asset, AssetPath
from pantomath_sdk.model.job.job import Job
from pantomath_sdk.enums.job_type import JobTypes
from pantomath_sdk.enums.platform_types import PlatformTypes


class FivetranConnectorConstructor(Job):
    """FivetranConnectorConstructor's Job Class
    used for getting the required infomation for Pantomath
    :param name: Name of the Constructor
    :type name: str
    ...
    """

    def __init__(self, name, assets: Union[List[Asset], None] = None):
        """Constructor method"""
        self._name = name
        self._asset_path = AssetPath(
            platform_type=self.get_platform_type(),
            connection_name="Fivetran Connection",
            assets=assets,
        )

    @staticmethod
    def create(name, assets: Union[List[Asset], None] = None):
        """Static method for obtaining FivetranConnectorConstructor's DataSet Class
        used for getting the required infomation for Pantomath
        :param name: Name of the Constructor
        :type name: str
        ...
        :return: FivetranConnectorConstructor class object
        :rtype: FivetranConnectorConstructor
        """
        return FivetranConnectorConstructor(name, assets)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return JobTypes.FIVETRAN_CONNECTOR.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return str(self._name)

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.FIVETRAN.value

    def get_asset_path(self):
        """Returns the asset path of the object
        ...
        :return: the asset path of the object
        :rtype: list
        """
        return self._asset_path



from pantomath_sdk.model.dataset.dataset import DataSet
from pantomath_sdk.enums.dataset_type import DataSetTypes
from pantomath_sdk.enums.platform_types import PlatformTypes


class PowerBIDashboard(DataSet):
    """PowerBIDashboard's DataSet Class used for getting the required infomation for Pantomath
    :param dashboard_id: The ID of the Dashboard
    :type dashboard_id: str
    :param workspace_id: The ID of the Workspace
    :type workspace_id: str
    :param name: The name of the Dashboard
    :type name: str
    ...
    """

    def __init__(self, dashboard_id, workspace_id, name):
        """Constructor method"""
        self._dashboard_id = dashboard_id
        self._workspace_id = workspace_id
        self._name = name

    @staticmethod
    def create(dashboard_id, workspace_id, name):
        """Static method for obtaining PowerBIDashboard's DataSet Class
        used for getting the required infomation for Pantomath
        :param dashboard_id: The ID of the Dashboard
        :type dashboard_id: str
        :param workspace_id: The ID of the Workspace
        :type workspace_id: str
        :param name: The name of the Dashboard
        :type name: str
        ...
        :return: PowerBIDashboard class object
        :rtype: PowerBIDashboard
        """
        return PowerBIDashboard(dashboard_id, workspace_id, name)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return DataSetTypes.POWER_BI_DASHBOARD.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        fqn = str(
            "app.powerbi.com/groups/"
            + str(self._workspace_id)
            + "/dashboards/"
            + str(self._dashboard_id)
        ).lower()
        return fqn

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.POWERBI.value



from pantomath_sdk import SynapsePipeline, PlatformTypes, AssetPathTypes, Asset


def test_synapse_pipeline_good():
    node = SynapsePipeline(
        name="SynapsePipeline Unit Test", pipeline_id="3674798497356"
    )
    assert node.get_name() == "SynapsePipeline Unit Test"
    assert node.get_type() == "PIPELINE"
    assert node.get_fully_qualified_object_name() == "3674798497356"
    assert node.get_platform_type() == PlatformTypes.SYNAPSE.value

    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "SYNAPSE", "type": "PLATFORM"}, {"depth": 1, "name": "Synapse Connection", "type": "CONNECTION"}]'
    )


def test_custom_synapse_pipeline_asset_path_good():
    node = SynapsePipeline(
        name="SynapsePipeline Unit Test",
        pipeline_id="3674798497356",
        assets=[Asset(name="Factory", type=AssetPathTypes.FACTORY.value)],
    )

    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "SYNAPSE", "type": "PLATFORM"}, {"depth": 1, "name": "Synapse Connection", "type": "CONNECTION"}, {"depth": 2, "name": "Factory", "type": "FACTORY"}]'
    )



from pantomath_sdk.client import *
from pantomath_sdk.enums import *
from pantomath_sdk.model import *
from pantomath_sdk.request import *
from pantomath_sdk.util import *
from pantomath_sdk.sdk import *



from pantomath_sdk import FTP, PlatformTypes


def test_ftp_good():
    test_node = FTP(uri="https://testing.dev/ftp/", name="ftp Unit Test")
    assert test_node.get_name() == "ftp Unit Test"
    assert test_node.get_type() == "FTP"
    assert test_node.get_fully_qualified_object_name() == "https://testing.dev/ftp/"
    assert test_node.get_platform_type() == PlatformTypes.UNKNOWN.value



from pantomath_sdk import S3Bucket, PlatformTypes


def test_s3_bucket_good():
    test_node = S3Bucket(s3_bucket="S3 Bucket Unit Test")
    assert test_node.get_name() == "S3 Bucket Unit Test"
    assert test_node.get_type() == "S3_BUCKET"
    assert test_node.get_fully_qualified_object_name() == "S3 Bucket Unit Test"
    assert test_node.get_platform_type() == PlatformTypes.S3.value



from pantomath_sdk import ADFActivity, PlatformTypes, Asset, AssetPathTypes


def test_adf_activity_good():
    node = ADFActivity(
        pipeline_id=13465478579, name="ADFActivity Unit Test", assets=None
    )
    assert node.get_name() == "ADFActivity Unit Test"
    assert node.get_type() == "ADF_ACTIVITY"
    assert (
        node.get_fully_qualified_object_name()
        == "13465478579/activities/adfactivity%20unit%20test"
    )
    assert node.get_platform_type() == PlatformTypes.ADF.value
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "ADF", "type": "PLATFORM"}, {"depth": 1, "name": "Azure Data Factory Connection", "type": "CONNECTION"}]'
    )


def test_custom_asset_path_adf_activity_good():
    node = ADFActivity(
        pipeline_id=13465478579,
        name="ADFActivity Unit Test",
        assets=[Asset("Channel A", AssetPathTypes.CHANNEL.value)],
    )

    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "ADF", "type": "PLATFORM"}, {"depth": 1, "name": "Azure Data Factory Connection", "type": "CONNECTION"}, {"depth": 2, "name": "Channel A", "type": "CHANNEL"}]'
    )



from pantomath_sdk import PowerBIRefresh, PlatformTypes
from pantomath_sdk.enums.asset_path import AssetPathTypes
from pantomath_sdk.model.asset_path.asset_path import Asset


def test_power_bi_refresh_good():
    node = PowerBIRefresh(
        name="PowerBIRefresh Unit Test",
        refresh_schedule_context="refresh_schedule_context",
        dataset_id="5785678976809",
    )
    assert node.get_name() == "PowerBIRefresh Unit Test"
    assert node.get_type() == "POWER_BI_REFRESH"
    assert (
        node.get_fully_qualified_object_name()
        == "refresh_schedule_context/5785678976809/powerbirefresh_unit_test"
    )
    assert node.get_platform_type() == PlatformTypes.POWERBI.value

    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "POWERBI", "type": "PLATFORM"}, {"depth": 1, "name": "PowerBI Connection", "type": "CONNECTION"}]'
    )


def test_custom_power_bi_refresh_asset_path_good():
    node = PowerBIRefresh(
        name="PowerBIRefresh Unit Test",
        refresh_schedule_context="refresh_schedule_context",
        dataset_id="5785678976809",
        assets=[Asset(name="Factory", type=AssetPathTypes.FACTORY.value)],
    )
    assert (
        node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "POWERBI", "type": "PLATFORM"}, {"depth": 1, "name": "PowerBI Connection", "type": "CONNECTION"}, {"depth": 2, "name": "Factory", "type": "FACTORY"}]'
    )



from pantomath_sdk import DataikuRecipe, PlatformTypes
from pantomath_sdk.enums.asset_path import AssetPathTypes
from pantomath_sdk.model.asset_path.asset_path import Asset


def test_dataiku_model_good():
    test_node = DataikuRecipe(
        host_name="host",
        project_key="96868946864896189",
        recipe_name="DataikuRecipe Unit Test",
    )
    assert test_node.get_name() == "DataikuRecipe Unit Test"
    assert test_node.get_type() == "RECIPE"
    assert (
        test_node.get_fully_qualified_object_name()
        == "host/projects/96868946864896189/recipes/%24dataikurecipe%20unit%20test"
    )
    assert test_node.get_platform_type() == PlatformTypes.DATAIKU.value
    assert (
        test_node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "DATAIKU", "type": "PLATFORM"}, {"depth": 1, "name": "host", "type": "CONNECTION"}]'
    )


def test_custom_dataiku_project_asset_path_good():
    test_node = DataikuRecipe(
        host_name="host",
        project_key="96868946864896189",
        recipe_name="DataikuRecipe Unit Test",
        assets=[
            Asset("Factory", AssetPathTypes.FACTORY.value),
        ],
    )
    assert (
        test_node.get_asset_path().__str__()
        == '[{"depth": 0, "name": "DATAIKU", "type": "PLATFORM"}, {"depth": 1, "name": "host", "type": "CONNECTION"}, {"depth": 2, "name": "Factory", "type": "FACTORY"}]'
    )



from pantomath_sdk import DataikuModel, PlatformTypes


def test_dataiku_model_good():
    test_node = DataikuModel(
        host_name="host",
        project_key="96868946864896189",
        model_name="DataikuModel Unit Test",
        model_id=3456745457,
    )
    assert test_node.get_name() == "DataikuModel Unit Test"
    assert test_node.get_type() == "MODEL"
    assert (
        test_node.get_fully_qualified_object_name()
        == "host/project/96868946864896189/models/dataikumodel%20unit%20test"
    )
    assert test_node.get_platform_type() == PlatformTypes.DATAIKU.value



from pantomath_sdk import PowerBIReport, PlatformTypes


def test_powerbi_report_good():
    test_node = PowerBIReport(
        workspace_id=123124124,
        report_id=9879789789,
        name="PowerBIReport Unit Test",
    )
    assert test_node.get_name() == "PowerBIReport Unit Test"
    assert test_node.get_type() == "POWER_BI_REPORT"
    assert (
        test_node.get_fully_qualified_object_name()
        == "app.powerbi.com/groups/123124124/reports/9879789789/reportsection"
    )
    assert test_node.get_platform_type() == PlatformTypes.POWERBI.value



from pantomath_sdk import PowerBiDataFlowEntity, PlatformTypes


def test_powerbi_dataflow_entity_good():
    test_node = PowerBiDataFlowEntity(
        group_id=123124124,
        object_id=9879789789,
        name="powerbi_dataflow_entity Unit Test",
    )
    assert test_node.get_name() == "powerbi_dataflow_entity Unit Test"
    assert test_node.get_type() == "POWER_BI_DATAFLOW_ENTITY"
    assert (
        test_node.get_fully_qualified_object_name()
        == "app.powerbi.com/groups/123124124/dataflows/9879789789/syntheticentity/"
        "powerbi_dataflow_entity unit test"
    )
    assert test_node.get_platform_type() == PlatformTypes.POWERBI.value



from pantomath_sdk.enums.asset_path import AssetPathTypes
from typing import List, Union
import json


class Asset:
    def __init__(self, name: str, type: AssetPathTypes):
        self.name = name
        self.type = type


class AssetPathObject:
    def __init__(self, depth: int, name: str, type: AssetPathTypes):
        self.depth = depth
        self.name = name
        self.type = type


class AssetPath:
    def __init__(
        self,
        platform_type: str,
        connection_name: str,
        assets: Union[List[Asset], None] = None,
    ):
        self.assets: List[AssetPathObject] = [
            AssetPathObject(
                depth=0, name=platform_type, type=AssetPathTypes.PLATFORM.value
            ),
            AssetPathObject(
                depth=1, name=connection_name, type=AssetPathTypes.CONNECTION.value
            ),
        ]

        if assets is not None:
            for asset in assets:
                self.append_asset(asset.name, asset.type)

    @property
    def get_path(self) -> str:
        return "/".join(
            asset.name for asset in sorted(self.assets, key=lambda x: x.depth)
        )

    @property
    def get_asset_paths(self) -> List[AssetPathObject]:
        return self.assets

    def __str__(self) -> str:
        return json.dumps([asset.__dict__ for asset in self.assets])

    def append_asset(self, name: str, type: AssetPathTypes) -> "AssetPath":
        current_depth = len(self.assets)
        asset = AssetPathObject(depth=current_depth, name=name, type=type)
        self.assets.append(asset)
        return self

    def get_asset_path(self):
        asset_paths = []
        for asset in self.assets:
            asset_paths.append(
                {
                    "depth": asset.depth,
                    "name": asset.name,
                    "type": asset.type,
                }
            )
        return asset_paths



from aenum import Enum


class JobDidNotRunAtMethods(Enum):
    """This class is an enum of all the JobDidNotRunAtMethods
    supported by the Pantomath Python SDK."""

    TRIGGER_TYPE = "TRIGGER_TYPE"
    NEXT_RUN_AT = "NEXT_RUN_AT"

    @staticmethod
    def get_job_did_not_run_at_method():
        """Get a list of all the enum values
        ...
        :return: list of all the enum values
        :rtype: list of strings
        """
        return dir(JobDidNotRunAtMethods)[: len(JobDidNotRunAtMethods)]

    @staticmethod
    def is_job_did_not_run_at_method(input):
        """Validates if the unput is part of the enum
        :param input: string to check if part of enum
        :type input: str
        ...
        :return: True if in enum else False
        :rtype: Boolean
        """
        return input in JobDidNotRunAtMethods.get_job_did_not_run_at_method()



from pantomath_sdk import TableauWorkbook, PlatformTypes


def test_tableau_workbook_good():
    test_node = TableauWorkbook(
        host="host",
        uri="uri",
        name="TableauWorkbook Unit Test",
    )
    assert test_node.get_name() == "TableauWorkbook Unit Test"
    assert test_node.get_type() == "TABLEAU_WORKBOOK"
    assert test_node.get_fully_qualified_object_name() == "host/uri"
    assert test_node.get_platform_type() == PlatformTypes.TABLEAU.value



from aenum import Enum


class PlatformTypes(Enum):
    """This class is an enum of all the PlatformTypes supported by the Pantomath Python SDK."""

    ADF = "ADF"
    AIRFLOW = "AIRFLOW"
    ALTERYX = "ALTERYX"
    ANALYZER = "ANALYZER"
    API_GATEWAY = "API_GATEWAY"
    ASTRO = "ASTRO"
    ASTRONOMER = "ASTRONOMER"
    AWS = "AWS"
    AWS_ATHENA = "AWS_ATHENA"
    AWS_EMR = "AWS_EMR"
    AWS_GLUE = "AWS_GLUE"
    AWS_QUICKSIGHT = "AWS_QUICKSIGHT"
    AWS_SAGEMAKER = "AWS_SAGEMAKER"
    AZSTORAGE = "AZSTORAGE"
    BIGQUERY = "BIGQUERY"
    BIG_QUERY = "BIG_QUERY"
    BOX = "BOX"
    CASSANDRADB = "CASSANDRADB"
    CLICKHOUSE = "CLICKHOUSE"
    COMPOSER = "COMPOSER"
    CONTROL_M = "CONTROL_M"
    CUSTOM_LOGS = "CUSTOM_LOGS"
    DATAIKU = "DATAIKU"
    DATAPROC = "DATAPROC"
    DATASTAGE = "DATASTAGE"
    DATA_ACTIVATOR = "DATA_ACTIVATOR"
    DATA_PROC = "DATA_PROC"
    DBT = "DBT"
    DBT_CORE = "DBT_CORE"
    DBX = "DBX"
    DBX_LEGACY = "DBX_LEGACY"
    DELTA_LAKE = "DELTA_LAKE"
    DOMO = "DOMO"
    DROPBOX = "DROPBOX"
    DYNAMODB = "DYNAMODB"
    EXCEL = "EXCEL"
    EXTERNAL = "EXTERNAL"
    FIREBASE = "FIREBASE"
    FIVETRAN = "FIVETRAN"
    GOOGLE_DATA_FUSION = "GOOGLE_DATA_FUSION"
    GOOGLE_DRIVE = "GOOGLE_DRIVE"
    GOOGLE_LOOKER = "GOOGLE_LOOKER"
    GOOGLE_STORAGE = "GOOGLE_STORAGE"
    GREENPLUM = "GREENPLUM"
    HPCC = "HPCC"
    HVR = "HVR"
    IBM_DATASTAGE = "IBM_DATASTAGE"
    INFORMATICA = "INFORMATICA"
    KAFKA = "KAFKA"
    KINESIS = "KINESIS"
    KNIME = "KNIME"
    LOGIC_APP = "LOGIC_APP"
    LOGIC_APPS = "LOGIC_APPS"
    MARIADB = "MARIADB"
    MATILLION = "MATILLION"
    METABASE = "METABASE"
    MICROSOFT_FABRIC = "MICROSOFT_FABRIC"
    MICROSTRATEGY = "MICROSTRATEGY"
    MODE = "MODE"
    MONGODB = "MONGODB"
    MYSQL = "MYSQL"
    NEO4J = "NEO4J"
    OBIEE = "OBIEE"
    ONELAKE = "ONELAKE"
    ORACLE = "ORACLE"
    ORACLE_DATA_INTEGRATOR = "ORACLE_DATA_INTEGRATOR"
    OUTLOOK = "OUTLOOK"
    PBIRS = "PBIRS"
    POSTGRES = "POSTGRES"
    POWERBI = "POWERBI"
    POWER_APPS = "POWER_APPS"
    POWER_AUTOMATE = "POWER_AUTOMATE"
    PREFECT = "PREFECT"
    QLIKSENSE = "QLIKSENSE"
    QLIKVIEW = "QLIKVIEW"
    QLIK_REPLICATE = "QLIK_REPLICATE"
    RDS = "RDS"
    REDSHIFT = "REDSHIFT"
    S3 = "S3"
    SAP_BO = "SAP_BO"
    SAP_HANA = "SAP_HANA"
    SHAREPOINT = "SHAREPOINT"
    SIGMA = "SIGMA"
    SISENSE = "SISENSE"
    SNAP_LOGIC = "SNAP_LOGIC"
    SNOWFLAKE = "SNOWFLAKE"
    SPARK = "SPARK"
    SPOTFIRE = "SPOTFIRE"
    SQL_SERVER = "SQL_SERVER"
    SSAS = "SSAS"
    SSIS = "SSIS"
    SSRS = "SSRS"
    STARBURST = "STARBURST"
    SYNAPSE = "SYNAPSE"
    TABLEAU = "TABLEAU"
    TALEND = "TALEND"
    THOUGHTSPOT = "THOUGHTSPOT"
    UNKNOWN = "UNKNOWN"
    YELLOWBRICK = "YELLOWBRICK"

    @staticmethod
    def get_platform_types():
        """Get a list of all the enum values
        ...
        :return: list of all the enum values
        :rtype: list of strings
        """
        return dir(PlatformTypes)[: len(PlatformTypes)]

    @staticmethod
    def is_platform_type(input):
        """Validates if the unput is part of the enum
        :param input: string to check if part of enum
        :type input: str
        ...
        :return: True if in enum else False
        :rtype: Boolean
        """
        return input in PlatformTypes.get_platform_types()



from pantomath_sdk import JobTriggerTypes


def test_get_job_trigger_types():
    expected_list = ["SCHEDULE", "MANUAL", "UNKNOWN"]
    statuses = JobTriggerTypes.get_job_trigger_types()
    assert statuses.sort() == expected_list.sort()


def test_is_job_trigger_type():
    assert JobTriggerTypes.is_job_trigger_type("MANUAL")


def test_not_is_job_trigger_type():
    assert not JobTriggerTypes.is_job_trigger_type("FOO")



from pantomath_sdk.model.dataset.dataset import DataSet
from pantomath_sdk.enums.dataset_type import DataSetTypes
from pantomath_sdk.enums.platform_types import PlatformTypes


class FTP(DataSet):
    """File Transfer Protocol (FTP)'s DataSet Class
    used for getting the required infomation for Pantomath
    :param uri: uri of the FTP
    :type uri: str
    :param name: Name of the FTP
    :type name: str
    ...
    """

    def __init__(self, uri, name):
        """Constructor method"""
        self._uri = uri
        self._name = name

    @staticmethod
    def create(uri, name):
        """Static method for obtaining File Transfer Protocol's DataSet Class
        used for getting the required infomation for Pantomath
        :param uri: uri of the FTP
        :type uri: str
        :param name: Name of the FTP
        :type name: str
        ...
        :return: File Transfer Protocol class object
        :rtype: File Transfer Protocol
        """
        return FTP(uri, name)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return DataSetTypes.FTP.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return self._uri

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.UNKNOWN.value



from typing import Union, List
from pantomath_sdk.enums.asset_path import AssetPathTypes
from pantomath_sdk.model.asset_path.asset_path import Asset, AssetPath
from pantomath_sdk.model.job.job import Job
from pantomath_sdk.enums.job_type import JobTypes
from pantomath_sdk.enums.platform_types import PlatformTypes
import pantomath_sdk.util as util


class ControlMJob(Job):
    """ControlMJob's Job Class used for getting the required infomation for Pantomath
    :param host: host of the control m
    :type host: str
    :param server: name of the server
    :type server: str
    :param name: Name of the Job
    :type name: str
    :param folder_hierarchy: List of the Folders in order parent folder
    down to folder before this folder
    :type folder_hierarchy: list of strings
    ...
    """

    def __init__(
        self,
        host,
        server,
        name,
        folder_hierarchy=None,
        assets: Union[List[Asset], None] = None,
    ):
        """Constructor method"""
        assets = []
        if server is not None:
            assets.append(Asset(name=server, type=AssetPathTypes.SERVER.value))

        if folder_hierarchy is not None:
            for folder in folder_hierarchy:
                assets.append(Asset(name=folder, type=AssetPathTypes.FOLDER.value))

        self._host = host
        self._server = server
        self._folder_hierarchy = folder_hierarchy
        self._name = name
        self._asset_path = AssetPath(
            platform_type=PlatformTypes.CONTROL_M.value,
            connection_name=host,
            assets=assets,
        )

    @staticmethod
    def create(
        host, server, name, folder_hierarchy, assets: Union[List[Asset], None] = None
    ):
        """Static method for obtaining ControlMJob's DataSet Class
        used for getting the required infomation for Pantomath
        :param host: host of the control m
        :type host: str
        :param server: name of the server
        :type server: str
        :param name: Name of the Job
        :type name: str
        :param folder_hierarchy: List of the Folders in order parent folder
        down to folder before this folder
        :type folder_hierarchy: list of strings
        ...
        :return: ControlMJob class object
        :rtype: ControlMJob
        """
        return ControlMJob(host, server, name, folder_hierarchy, assets)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return JobTypes.JOB.value

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.CONTROL_M.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        hierarchy = (
            ("." + ".".join(self._folder_hierarchy))
            if self._folder_hierarchy is not None
            else ""
        )
        return util.sanitize_fully_qualified_object_name(
            self._host + "." + self._server + hierarchy + "." + self.get_name()
        )

    def get_asset_path(self) -> AssetPath:
        """Returns the Asset Path of the object
        ...
        :return: the Asset Path of the object
        :rtype: AssetPath
        """
        return self._asset_path



from pantomath_sdk.model.job.job import Job
from pantomath_sdk.enums.job_type import JobTypes
from pantomath_sdk.enums.platform_types import PlatformTypes
from typing import Union, List
from pantomath_sdk.model.asset_path.asset_path import Asset, AssetPath


class SynapseActivity(Job):
    """SynapseActivity's Job Class used for getting the required infomation for Pantomath
    :param pipeline_id: The Pipeline ID
    :type pipeline_id: str
    :param name: Name of the activity
    :type name: str
    ...
    """

    def __init__(self, pipeline_id, name, assets: Union[List[Asset], None] = None):
        """Constructor method"""
        self._pipeline_id = pipeline_id
        self._name = name
        self._asset_path = AssetPath(
            platform_type=PlatformTypes.SYNAPSE.value,
            connection_name="Synapse Connection",
            assets=assets,
        )

    @staticmethod
    def create(pipeline_id, name, assets: Union[List[Asset], None] = None):
        """Static method for obtaining SynapseActivity's DataSet Class
        used for getting the required infomation for Pantomath
        :param pipeline_id: The Pipeline ID
        :type pipeline_id: str
        :param name: Name of the activity
        :type name: str
        ...
        :return: SynapseActivity class object
        :rtype: SynapseActivity
        """
        return SynapseActivity(pipeline_id, name, assets)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return JobTypes.ACTIVITY.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return str(self._pipeline_id + "/activities/" + self._name).lower()

    def get_pipeline_id(self):
        """Returns the pipeline ID of the object
        ...
        :return: the pipeline ID of the object
        :rtype: str
        """
        return self._pipeline_id

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.SYNAPSE.value

    def get_asset_path(self):
        """Returns the asset path of the object
        ...
        :return: the asset path of the object
        :rtype: AssetPath
        """
        return self._asset_path



from pantomath_sdk.model.job.job import Job
from pantomath_sdk.enums.job_type import JobTypes
from pantomath_sdk.enums.platform_types import PlatformTypes
from typing import Union, List
from pantomath_sdk.model.asset_path.asset_path import Asset, AssetPath


class PowerBIRefresh(Job):
    """PowerBIRefresh's Job Class used for getting the required infomation for Pantomath
    :param dataset_id: The dataset ID
    :type dataset_id: str
    :param refresh_schedule_context: Name of the refresh_schedule_context
    :type refresh_schedule_context: str
    :param name: Name of the activity
    :type name: str
    ...
    """

    def __init__(
        self,
        dataset_id,
        refresh_schedule_context,
        name,
        assets: Union[List[Asset], None] = None,
    ):
        """Constructor method"""
        self._name = name
        self._refresh_schedule_context = refresh_schedule_context
        self._dataset_id = dataset_id
        self._asset_path = AssetPath(
            platform_type=PlatformTypes.POWERBI.value,
            connection_name="PowerBI Connection",
            assets=assets,
        )

    @staticmethod
    def create(
        dataset_id,
        refresh_schedule_context,
        name,
        assets: Union[List[Asset], None] = None,
    ):
        """Static method for obtaining PowerBIRefresh's DataSet Class
        used for getting the required infomation for Pantomath
        :param dataset_id: The dataset ID
        :type dataset_id: str
        :param refresh_schedule_context: Name of the refresh_schedule_context
        :type refresh_schedule_context: str
        :param name: Name of the activity
        :type name: str
        ...
        :return: PowerBIRefresh class object
        :rtype: PowerBIRefresh
        """
        return PowerBIRefresh(dataset_id, refresh_schedule_context, name, assets)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return JobTypes.POWER_BI_REFRESH.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return (
            str(
                self._refresh_schedule_context
                + "/"
                + self._dataset_id
                + "/"
                + self._name
            ).replace(" ", "_")
        ).lower()

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.POWERBI.value

    def get_asset_path(self):
        """Returns the asset path of the object
        ...
        :return: the asset path of the object
        :rtype: AssetPath
        """
        return self._asset_path



from typing import Union, List
from pantomath_sdk.model.asset_path.asset_path import Asset, AssetPath
from pantomath_sdk.model.job.job import Job
from pantomath_sdk.enums.job_type import JobTypes
from pantomath_sdk.enums.platform_types import PlatformTypes
import pantomath_sdk.util as util


class ADFActivity(Job):
    """ADFActivity's Job Class used for getting the required infomation for Pantomath
    :param pipeline_id: The Pipeline ID
    :type pipeline_id: str
    :param name: Name of the activity
    :type name: str
    ...
    """

    def __init__(self, pipeline_id, name, assets: Union[List[Asset], None] = None):
        """Constructor method"""
        self._pipeline_id = pipeline_id
        self._name = name
        self.asset_path = AssetPath(
            platform_type=PlatformTypes.ADF.value,
            connection_name="Azure Data Factory Connection",
            assets=assets,
        )

    @staticmethod
    def create(pipeline_id, name, assets: Union[List[Asset], None] = None):
        """Static method for obtaining ADFActivity's DataSet Class
        used for getting the required infomation for Pantomath
        :param pipeline_id: The Pipeline ID
        :type pipeline_id: str
        :param name: Name of the activity
        :type name: str
        ...
        :return: ADFActivity class object
        :rtype: ADFActivity
        """
        return ADFActivity(pipeline_id, name, assets)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return JobTypes.ADF_ACTIVITY.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return util.sanitize_fully_qualified_object_name(
            str(str(self._pipeline_id) + "/activities/" + str(self._name))
        )

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.ADF.value

    def get_asset_path(self) -> AssetPath:
        """Returns the asset path of the object
        ...
        :return: the asset path of the object
        :rtype: AssetPath
        """
        return self.asset_path



from pantomath_sdk.model.job.job import Job
from pantomath_sdk.enums.job_type import JobTypes
from pantomath_sdk.enums.platform_types import PlatformTypes
from typing import Union, List
from pantomath_sdk.model.asset_path.asset_path import Asset, AssetPath


class PowerBiDataFlow(Job):
    """PowerBiDataFlow's Job Class used for getting the required infomation for Pantomath
    :param group_id: The Group ID
    :type group_id: str
    :param object_id: The Object ID
    :type object_id: str
    :param name: Name of the Data  Flow
    :type name: str
    ...
    """

    def __init__(
        self, group_id, object_id, name, assets: Union[List[Asset], None] = None
    ):
        """Constructor method"""
        self._name = name
        self._object_id = object_id
        self._groupId = group_id
        self._asset_path = AssetPath(
            platform_type=PlatformTypes.POWERBI.value,
            connection_name="PowerBI Connection",
            assets=assets,
        )

    @staticmethod
    def create(group_id, object_id, name, assets: Union[List[Asset], None] = None):
        """Static method for obtaining PowerBiDataFlow's DataSet Class
        used for getting the required infomation for Pantomath
        :param group_id: The Group ID
        :type group_id: str
        :param object_id: The Object ID
        :type object_id: str
        :param name: Name of the Data  Flow
        :type name: str
        ...
        :return: PowerBiDataFlow class object
        :rtype: PowerBiDataFlow
        """
        return PowerBiDataFlow(group_id, object_id, name, assets)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return JobTypes.POWER_BI_DATAFLOW.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return str(
            "app.powerbi.com/groups/" + self._groupId + "/dataflows/" + self._object_id
        ).lower()

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.POWERBI.value

    def get_asset_path(self) -> AssetPath:
        """Returns the asset path of the object
        ...
        :return: the asset path of the object
        :rtype: AssetPath
        """
        return self._asset_path



from pantomath_sdk import AstroDag
from pantomath_sdk.enums.platform_types import PlatformTypes


def test_astro_dag_good():
    test_node = AstroDag(
        host_name="https://testing.dev/astro_dags/", name="Astro Unit Test"
    )
    assert test_node.get_name() == "Astro Unit Test"
    assert test_node.get_type() == "DAG"
    assert (
        test_node.get_fully_qualified_object_name()
        == "https%3a//testing.dev/astro_dags//astro%20unit%20test"
    )
    assert test_node.get_platform_type() == PlatformTypes.ASTRO.value



from pantomath_sdk import AdfDataFlow, PlatformTypes


def test_adf_data_flow_good():
    test_node = AdfDataFlow(data_flow_id="96868946864896189", name="ADF Unit Test")
    assert test_node.get_name() == "ADF Unit Test"
    assert test_node.get_type() == "ADF_DATA_FLOW"
    assert test_node.get_fully_qualified_object_name() == "96868946864896189"
    assert test_node.get_platform_type() == PlatformTypes.ADF.value



from typing import Union, List
from pantomath_sdk.enums.platform_types import PlatformTypes
from pantomath_sdk.model.asset_path.asset_path import Asset
from pantomath_sdk.model.job.job import Job
from pantomath_sdk.enums.job_type import JobTypes
from pantomath_sdk.model.asset_path import AssetPath
import pantomath_sdk.util as util


class AstroTask(Job):
    """AstroTask's Job Class used for getting the required infomation for Pantomath
    :param dag_name: Name of the dag
    :type dag_name: str
    :param host_name: Name of the host
    :type host_name: str
    :param name: Name of the task
    :type name: str
    ...
    """

    def __init__(
        self, dag_name, host_name, name, assets: Union[List[Asset], None] = None
    ):
        """Constructor method"""
        self._dag_name = dag_name
        self._host_name = host_name
        self._name = name
        self.asset_path = AssetPath(
            platform_type=PlatformTypes.ASTRONOMER.value,
            connection_name=self._host_name,
            assets=assets,
        )

    @staticmethod
    def create(dag_name, host_name, name, assets: Union[List[Asset], None] = None):
        """Static method for obtaining AstroTask's DataSet Class
        used for getting the required infomation for Pantomath
        :param dag_name: Name of the dag
        :type dag_name: str
        :param host_name: Name of the host
        :type host_name: str
        :param name: Name of the task
        :type name: str
        ...
        :return: AstroTask class object
        :rtype: AstroTask
        """
        return AstroTask(dag_name, host_name, name, assets)

    def get_name(self):
        """Returns the name of the object
        ...
        :return: the name of the object
        :rtype: str
        """
        return self._name

    def get_type(self):
        """Returns the type of the object
        ...
        :return: the type of the object
        :rtype: str
        """
        return JobTypes.TASK.value

    def get_fully_qualified_object_name(self):
        """Returns the Fully Qualified Name of the object
        ...
        :return: the Fully Qualified Name of the object
        :rtype: str
        """
        return util.sanitize_fully_qualified_object_name(
            self._host_name + "/" + self._dag_name + "/" + self.get_name()
        )

    def get_platform_type(self):
        """Returns the platform type of the object
        ...
        :return: the platform type of the object
        :rtype: str
        """
        return PlatformTypes.ASTRO.value

    def get_asset_path(self) -> AssetPath:
        """Returns the asset path of the object
        ...
        :return: the asset path of the object
        :rtype: AssetPath
        """
        return self.asset_path



from pantomath_sdk import PostJobRunLog
import pytest


def test_logs_good_data():
    mock_data_set = {
        "object_name": "objectName",
        "object_type": "objectType",
        "fully_qualified_object_name": "fullyQualifiedObjectName",
    }
    log = {
        "job_run_id": "jobRunId",
        "object_name": "objectName",
        "object_type": "objectType",
        "fully_qualified_object_name": "fullyQualifiedObjectName",
        "asset_path": [
            {"depth": 0, "name": "ASTRONOMER", "type": "PLATFORM"},
            {"depth": 1, "name": "testHost", "type": "CONNECTION"},
        ],
        "status": "status",
        "message": "message",
        "records_effected": 42,
        "target_data_sets": [mock_data_set],
        "source_data_sets": [mock_data_set],
        "iso_timestamp": "isoTimestamp",
    }
    log_data = PostJobRunLog(**log)
    assert log_data.validate_data()


def test_log_data_bad_tdataset():
    mock_data_set = {
        "object_name": "objectName",
        "object_type": "objectType",
        "fully_qualified_object_name": "fullyQualifiedObjectName",
    }
    log = {
        "job_run_id": "jobRunId",
        "object_name": "objectName",
        "object_type": "objectType",
        "fully_qualified_object_name": "fullyQualifiedObjectName",
        "asset_path": [
            {"depth": 0, "name": "ASTRONOMER", "type": "PLATFORM"},
            {"depth": 1, "name": "testHost", "type": "CONNECTION"},
        ],
        "status": "status",
        "message": "message",
        "records_effected": 42,
        "target_data_sets": "BAD_DATA",
        "source_data_sets": [mock_data_set],
        "iso_timestamp": "isoTimestamp",
    }
    with pytest.raises(TypeError):
        PostJobRunLog(log)


def test_send_log_data_bad_sdataset():
    mock_data_set = {
        "object_name": "objectName",
        "object_type": "objectType",
        "fully_qualified_object_name": "fullyQualifiedObjectName",
    }
    log = {
        "job_run_id": "jobRunId",
        "object_name": "objectName",
        "object_type": "objectType",
        "fullyQualifiedObjectName": "fullyQualifiedObjectName",
        "asset_path": [
            {"depth": 0, "name": "ASTRONOMER", "type": "PLATFORM"},
            {"depth": 1, "name": "testHost", "type": "CONNECTION"},
        ],
        "status": "status",
        "message": "message",
        "records_effected": 42,
        "source_data_sets": "BAD_DATA",
        "target_data_sets": [mock_data_set],
        "iso_timestamp": "isoTimestamp",
    }
    with pytest.raises(TypeError):
        PostJobRunLog(log)


def test_send_log_data_no_optional():
    mock_data_set = {
        "object_name": "objectName",
        "object_type": "objectType",
        "fully_qualified_object_name": "fullyQualifiedObjectName",
    }
    log = {
        "job_run_id": "jobRunId",
        "object_name": "objectName",
        "object_type": "objectType",
        "fully_qualified_object_name": "fullyQualifiedObjectName",
        "asset_path": [
            {"depth": 0, "name": "ASTRONOMER", "type": "PLATFORM"},
            {"depth": 1, "name": "testHost", "type": "CONNECTION"},
        ],
        "status": "status",
        "message": "message",
        "source_data_sets": [mock_data_set],
        "target_data_sets": [mock_data_set],
        "iso_timestamp": "isoTimestamp",
    }
    with pytest.raises(TypeError):
        PostJobRunLog(log)


def test_send_log_data_bad_optional():
    mock_data_set = {
        "object_name": "objectName",
        "object_type": "objectType",
        "fully_qualified_object_name": "fullyQualifiedObjectName",
    }
    log = {
        "job_run_id": "jobRunId",
        "object_name": "objectName",
        "object_type": "objectType",
        "fully_qualified_object_name": "fullyQualifiedObjectName",
        "asset_path": [
            {"depth": 0, "name": "ASTRONOMER", "type": "PLATFORM"},
            {"depth": 1, "name": "testHost", "type": "CONNECTION"},
        ],
        "status": "status",
        "message": "message",
        "records_effected": "BAD_DATA",
        "source_data_sets": [mock_data_set],
        "target_data_sets": [mock_data_set],
        "iso_timestamp": "isoTimestamp",
    }
    with pytest.raises(TypeError):
        PostJobRunLog(log)


def test_send_log_data_missing_data():
    mock_data_set = {
        "object_name": "objectName",
        "object_type": "objectType",
        "fully_qualified_object_name": "fullyQualifiedObjectName",
    }
    log = {
        "object_name": "objectName",
        "object_type": "objectType",
        "fully_qualified_object_name": "fullyQualifiedObjectName",
        "asset_path": [
            {"depth": 0, "name": "ASTRONOMER", "type": "PLATFORM"},
            {"depth": 1, "name": "testHost", "type": "CONNECTION"},
        ],
        "status": "status",
        "message": "message",
        "records_effected": 42,
        "source_data_sets": [mock_data_set],
        "target_data_sets": [mock_data_set],
        "iso_timestamp": "isoTimestamp",
    }
    with pytest.raises(TypeError):
        PostJobRunLog(log)


def test_send_log_data_no_data():
    with pytest.raises(TypeError):
        PostJobRunLog(None)



from pantomath_sdk import PantomathApiClient
from pantomath_sdk import JobRunLogStatuses
import requests_mock
import pytest
import sys

mock_data_set = {
    "objectName": "objectName",
    "objectType": "objectType",
    "fullyQualifiedObjectName": "fullyQualifiedObjectName",
}

mock_asset_path_set = [
    {
        "depth": 0,
        "name": "name",
        "type": "type",
    },
    {
        "depth": 1,
        "name": "name",
        "type": "type",
    },
]


def assert_basic_request_data(mock, request):
    assert mock.called
    assert mock.call_count == 1
    assert request.method == "POST"
    assert "x-api-key" in request.headers
    assert "x-sdk-type" in request.headers
    assert request.headers["x-sdk-type"] == "Python"
    assert request.headers["x-api-key"] == "1234"


def test_post_job_run_logs_no_data():
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        client = PantomathApiClient(api_key="1234")
        client.post_job_run_logs(jobRunLogs=None)
        assert mock.call_count == 0


def test_post_job_run_logs_bad_data():
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        client = PantomathApiClient(api_key="1234")
        client.post_job_run_logs(jobRunLogs="test")
        assert mock.call_count == 0


def test_post_job_run_logs_good_data():
    log = {
        "jobRunId": "jobRunId",
        "objectName": "objectName",
        "objectType": "objectType",
        "fullyQualifiedObjectName": "fullyQualifiedObjectName",
        "assetPaths": {"assets": mock_asset_path_set},
        "status": JobRunLogStatuses.SUCCEEDED.value,
        "message": "message",
        "recordsEffected": 42,
        "targetDataSets": [mock_data_set],
        "sourceDataSets": [mock_data_set],
        "isoTimestamp": "isoTimestamp",
    }
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        client = PantomathApiClient(api_key="1234")
        client.post_job_run_logs(jobRunLogs=[log])
        request = mock.request_history[0]
        assert_basic_request_data(mock, request)
        sys.stdout.write(request.body)
        assert "jobRunLogs" in request.body


def test_post_job_run_logs_good_data_2():
    client = PantomathApiClient(api_key="1234")
    log = {
        "jobRunId": "jobRunId",
        "objectName": "objectName",
        "objectType": "objectType",
        "fullyQualifiedObjectName": "fullyQualifiedObjectName",
        "assetPaths": {"assets": mock_asset_path_set},
        "status": JobRunLogStatuses.SUCCEEDED.value,
        "message": "message",
        "recordsEffected": 42,
        "targetDataSets": [mock_data_set],
        "sourceDataSets": [mock_data_set],
        "isoTimestamp": "isoTimestamp",
    }
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        client = PantomathApiClient(api_key="1234")
        client.post_job_run_logs(jobRunLogs=[log, log, log])
        request = mock.request_history[0]
        assert_basic_request_data(mock, request)
        sys.stdout.write(request.body)
        assert "jobRunLogs" in request.body


def test_post_job_run_logs_one_bad_data():
    good_log = {
        "jobRunId": "jobRunId",
        "objectName": "objectName",
        "objectType": "objectType",
        "fullyQualifiedObjectName": "fullyQualifiedObjectName",
        "assetPaths": {"assets": mock_asset_path_set},
        "status": JobRunLogStatuses.SUCCEEDED.value,
        "message": "message",
        "recordsEffected": 42,
        "targetDataSets": [mock_data_set],
        "sourceDataSets": [mock_data_set],
        "isoTimestamp": "isoTimestamp",
    }
    bad_log = {}
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        client = PantomathApiClient(api_key="1234")
        with pytest.raises(TypeError):
            client.post_job_run_logs(jobRunLogs=[good_log, good_log, good_log, bad_log])
        assert mock.call_count == 0


def test_send_log_no_data():
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        client = PantomathApiClient(api_key="1234")
        client._send_logs(None)
        assert mock.call_count == 0


def test_send_logs_bad_data():
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        client = PantomathApiClient(api_key="1234")
        client._send_logs("test")
        assert mock.call_count == 1


def test_send_logs_good_data():
    log = {
        "jobRunId": "jobRunId",
        "objectName": "objectName",
        "objectType": "objectType",
        "fullyQualifiedObjectName": "fullyQualifiedObjectName",
        "assetPaths": {"assets": mock_asset_path_set},
        "status": JobRunLogStatuses.SUCCEEDED.value,
        "message": "message",
        "recordsEffected": 42,
        "targetDataSets": [mock_data_set],
        "sourceDataSets": [mock_data_set],
        "isoTimestamp": "isoTimestamp",
    }
    with requests_mock.Mocker() as mock:
        mock.post("https://api.dev-pantomath.org/v1/job-run-logs", status_code=200)
        client = PantomathApiClient(api_key="1234")
        client._send_logs([log])
        request = mock.request_history[0]
        assert_basic_request_data(mock, request)
        sys.stdout.write(request.body)
        assert "jobRunLogs" in request.body


def test_send_log_data_good_data():
    client = PantomathApiClient(api_key="1234")
    log = {
        "jobRunId": "jobRunId",
        "objectName": "objectName",
        "objectType": "objectType",
        "fullyQualifiedObjectName": "fullyQualifiedObjectName",
        "assetPaths": {"assets": mock_asset_path_set},
        "status": JobRunLogStatuses.SUCCEEDED.value,
        "message": "message",
        "recordsEffected": 42,
        "targetDataSets": [mock_data_set],
        "sourceDataSets": [mock_data_set],
        "isoTimestamp": "isoTimestamp",
    }
    result = client.validate_log_data(log)
    assert result


def test_send_log_data_bad_tdataset():
    client = PantomathApiClient(api_key="1234")
    log = {
        "jobRunId": "jobRunId",
        "objectName": "objectName",
        "objectType": "objectType",
        "fullyQualifiedObjectName": "fullyQualifiedObjectName",
        "assetPaths": {"assets": mock_asset_path_set},
        "status": JobRunLogStatuses.SUCCEEDED.value,
        "message": "message",
        "recordsEffected": 42,
        "targetDataSets": "BAD_DATA",
        "sourceDataSets": [mock_data_set],
        "isoTimestamp": "isoTimestamp",
    }
    result = client.validate_log_data(log)
    assert not result


def test_send_log_data_bad_sdataset():
    client = PantomathApiClient(api_key="1234")
    log = {
        "jobRunId": "jobRunId",
        "objectName": "objectName",
        "objectType": "objectType",
        "fullyQualifiedObjectName": "fullyQualifiedObjectName",
        "assetPaths": {"assets": mock_asset_path_set},
        "status": JobRunLogStatuses.SUCCEEDED.value,
        "message": "message",
        "recordsEffected": 42,
        "sourceDataSets": "BAD_DATA",
        "targetDataSets": [mock_data_set],
        "isoTimestamp": "isoTimestamp",
    }
    result = client.validate_log_data(log)
    assert not result


def test_send_log_data_no_optional():
    client = PantomathApiClient(api_key="1234")
    log = {
        "jobRunId": "jobRunId",
        "objectName": "objectName",
        "objectType": "objectType",
        "fullyQualifiedObjectName": "fullyQualifiedObjectName",
        "assetPaths": {"assets": mock_asset_path_set},
        "status": JobRunLogStatuses.SUCCEEDED.value,
        "message": "message",
        "sourceDataSets": [mock_data_set],
        "targetDataSets": [mock_data_set],
        "isoTimestamp": "isoTimestamp",
    }
    result = client.validate_log_data(log)
    assert result


def test_send_log_data_bad_optional():
    client = PantomathApiClient(api_key="1234")
    log = {
        "jobRunId": "jobRunId",
        "objectName": "objectName",
        "objectType": "objectType",
        "fullyQualifiedObjectName": "fullyQualifiedObjectName",
        "assetPaths": {"assets": mock_asset_path_set},
        "status": "status",
        "message": "message",
        "recordsEffected": "BAD_DATA",
        "sourceDataSets": [mock_data_set],
        "targetDataSets": [mock_data_set],
        "isoTimestamp": "isoTimestamp",
    }
    result = client.validate_log_data(log)
    assert not result


def test_send_log_data_missing_data():
    client = PantomathApiClient(api_key="1234")
    log = {
        "objectName": "objectName",
        "objectType": "objectType",
        "fullyQualifiedObjectName": "fullyQualifiedObjectName",
        "assetPaths": {"assets": mock_asset_path_set},
        "status": "status",
        "message": "message",
        "recordsEffected": 42,
        "sourceDataSets": [mock_data_set],
        "targetDataSets": [mock_data_set],
        "isoTimestamp": "isoTimestamp",
    }
    result = client.validate_log_data(log)

    assert not result


def test_send_log_data_no_data():
    client = PantomathApiClient(api_key="1234")
    result = client.validate_log_data(None)
    assert not result
